<?php
/**
 * Работа с настройками
 * 
 * @version  1.4.1
 */
# подключаем все необходимые файлы
require_once '../config.php';

# авторизация
if ( $login && $password ) {
	if ( !isset($_COOKIE['login']) ) die();
	if ( !isset($_COOKIE['password']) ) die();

	if ( !$_COOKIE['login'] || $_COOKIE['login'] !=  hash('sha256', $login) ) die();
	if ( !$_COOKIE['password'] || $_COOKIE['password'] != hash('sha256', $password) ) die();
}

# определяем, какое задание нужно выполнить
$task = get_req_var('task');

# выполняем задание
switch ($task) {
	
	
	//*****************//
	// ОБЩИЕ НАСТРОЙКИ //
	//*****************//
	/**
	 * Добавить/изменить параметр в настройках
	 *
	 * @param string $key - название параметр
	 * @param string $value - новое значение параметра
	 */
	case 'save_option':
		
		# получаем параметры
		$name = get_req_var('key');
		$value = get_req_var('value');
		
		# сохраняем
		$settings->save_option($name, $value);
		
		# выводим результат
		echo 'Данные сохранены!'; die();
		break;
		
		
	/**
	 * Удалить параметр из настроек
	 *
	 * @param string $key - название параметра
	 */
	case 'delete_option':
		
		# получаем параметры
		$name = get_req_var('key');
		
		# удаляем
		$settings->delete_option($name);
		
		# выводим результат
		echo 'Данные удалены!'; die();
		break;
		
		
	/**
	 * Добавить значение в настройку, которая представлена в виде массива
	 * 
	 * @param string $key - название параметра
	 * @param string $value - значение, которое надо добавить в массив
	 */
	case 'add_value_to_option':
		
		# получаем параметры
		$name = get_req_var('key');
		$value = get_req_var('value');
		
		# сохраняем
		$settings->add_value_to_option($name, $value);
		
		# выводим результат
		echo 'Данные сохранены!'; die();
		break;
		
		
	/**
	 * Удалить значение из настройки, которая представлена в виде массива
	 * 
	 * @param string $key - название параметра
	 * @param string $value - значение, которое надо удалить из массива
	 */
	case 'delete_value_from_option':
		
		# получаем параметры
		$name = get_req_var('key');
		$value = get_req_var('value');
		
		# удаляем
		$settings->delete_value_from_option($name, $value);
		
		# выводим результат
		echo 'Данные удалены!'; die();
		break;
		
		
	//*****************//
	// НАСТРОЙКИ САЙТА //
	//*****************//
	/**
	 * Добавить/изменить параметр сайта
	 *
	 * @param string $key - название параметр
	 * @param string $value - новое значение параметра
	 */
	case 'save_site':
		
		# получаем параметры
		$name = get_req_var('key');
		$value = get_req_var('value');
		$id = get_req_var('site_id');
		
		# сохраняем
		global $database;
		$database->query("UPDATE sites SET `$name` = '$value' WHERE `id` = $id");
		
		# выводим результат
		echo 'Данные сохранены!'; die();
		break;
		
		
	//*******************//
	// НАСТРОЙКИ ЗАДАНИЙ //
	//*******************//
	/**
	 * Добавить/изменить параметр в настройках одного задания
	 * 
	 * @param string $task_name - название задания
	 * @param string $key - название параметр
	 * @param string $value - новое значение параметра
	 */
	case 'save_task_option':
		
		# получаем параметры
		$task_name = get_req_var('task_name');
		$name = get_req_var('key');
		$value = get_req_var('value');
	
		# сохраняем
		$settings->save_task_option($task_name, $name, $value);
	
		# выводим результат
		echo 'Данные сохранены!'; die();
		break;
		
		
	/**
	* Добавить значение в настройку, которая представлена в виде массива
	* 
	* @param string $task_name - название задания
	* @param string $key - название параметра
	* @param string $value - значение, которое надо добавить в массив
	*/
	case 'add_value_to_task_option':
	
		$h = block('add_value_to_task_option');
		
		# получаем параметры
		$task_name = get_req_var('task_name');
		$name = get_req_var('key');
		$value = get_req_var('value');
		
		# сохраняем
		$settings->add_value_to_task_option($task_name, $name, $value);
		
		unblock($h);
	
		# выводим результат
		echo 'Данные сохранены!'; die();
		break;
		
		
	/**
	* Удалить значение из настройки, которая представлена в виде массива
	* 
	* @param string $task_name - название задания
	* @param string $key - название параметра
	* @param string $value - значение, которое надо удалить из массива
	*/
	case 'delete_value_from_task_option':
	
		$h = block('delete_value_from_task_option');
		
		# получаем параметры
		$task_name = get_req_var('task_name');
		$name = get_req_var('key');
		$value = get_req_var('value');
		
		# удаляем
		$settings->delete_value_from_task_option($task_name, $name, $value);
		
		unblock($h);
	
		# выводим результат
		echo 'Данные удалены!'; die();
		break;
		
		
	/**
	 * Получить все настройки указанного задания
	 * 
	 * @param string $task_name - название задания
	 */
	case 'get_task_options':
		
		# получаем параметры
		$task_name = get_req_var('task_name');
		
		# получаем настройки
		$options = $settings->task_options($task_name);
		
		# выводим результат
		echo json_encode($options); die();
		break;
		
		
	/**
	 * Сбросить настройки задания
	 */
	case 'reset_task':
		
		# получаем параметры
		$task_name = get_req_var('task_name');
		
		# удаляем настройки
		$settings->reset_task($task_name);
		
		break;
		
}
