<?php
/**
 * Драйвер к сервису распознавания каптчи http://antigate.com
 *
 * @version  1.1.0
 */
class Anticaptcha {
	
	private $api;
	private $source;
	
	private $parser;
	private $browser;
	
	public function __construct($api='') {
		global $settings;
		
		$this->source = $settings->antigate_source;
		$this->api = $api;
		
		if ( $api ) {
			$this->browser = new Curl();
			$this->parser = new Parser();
		}
		
		return true;
	}
	
	public function ya_detect($html) {
		if ( strpos($html, '<title>Ой!</title>')!==false ) return true;
		return false;
	}
	
	public function ya_recognize($html) {
		
		if ( !$this->api ) return false;
		
		// разбираем форму с каптчей
		$key = $this->parser->parse_string($html, '|name="key" value="(.*?)"|i');
		$retpath = $this->parser->parse_string($html, '|name="retpath" value="(.*?)"|i');
		$captcha_url = $this->parser->parse_string($html, '|<img class="image form__captcha"[^>]*src="(.*?)"|i');
		
		// сохраняем каптчу на диск
		$data = $this->browser->get_page($captcha_url);
		$filename = dirname(__FILE__).'/'.uniqid().'.jpg';
		file_put_contents($filename, $data);
		
		// распознаём каптчу
		$captcha = $this->recognize($filename, $this->api);
		
		// вводим каптчу
		$result = $this->browser->get_page("http://yandex.ru/checkcaptcha?key=$key&retpath=$retpath&rep=$captcha");
		unlink($filename);
		
		return $result;
	}
	
	/**
	 * Распознать каптчу
	 */
	public function recognize(
		$filename,
		$apikey,
		$is_verbose = false,
		$sendhost = "antigate.com",
		$rtimeout = 5,
		$mtimeout = 120,
		$is_phrase = 0,
		$is_regsense = 0,
		$is_numeric = 0,
		$min_len = 0,
		$max_len = 0,
		$is_russian = 0)
	{
		if ( !$apikey ) return false;
		
		if ( $this->source == 'rucaptcha' )
		{
			$result = $this->recognize_rucaptcha(
								$filename,
								$apikey,
								$is_verbose,
								$sendhost,
								$rtimeout,
								$mtimeout,
								$is_phrase,
								$is_regsense,
								$is_numeric,
								$min_len,
								$max_len,
								$is_russian);
		}
		else
		{
			$result = $this->recognize_antigate(
								$filename,
								$apikey,
								$is_verbose,
								$sendhost,
								$rtimeout,
								$mtimeout,
								$is_phrase,
								$is_regsense,
								$is_numeric,
								$min_len,
								$max_len,
								$is_russian);
		}
		
		return $result;
	}
	/**
	 * Распознать текст с картинки через antigate.com
	 * 
	 * $filename - полный путь к файлу
	 * $apikey   - ключ для работы
	 * $rtimeout - задержка между опросами статуса капчи
	 * $mtimeout - время ожидания ввода капчи
	 * 
	 * включить/выключить verbose mode (комментирование происходящего):
	 * $is_verbose - false(выключить),  true(включить)
	 * 
	 * дополнительно (дефолтные параметры править не нужно без необходимости):
	 * $is_phrase   - 0 либо 1 - флаг "в капче 2 и более слов"
	 * $is_regsense - 0 либо 1 - флаг "регистр букв в капче имеет значение"
	 * $is_numeric  - 0 либо 1 - флаг "капча состоит только из цифр"
	 * $min_len     - 0 (без ограничений), любая другая цифра указывает минимальную длину текста капчи
	 * $max_len     - 0 (без ограничений), любая другая цифра указывает максимальную длину текста капчи
	 * $is_russian  - 0 либо 1 - флаг "капча должна демонстрироваться только русскоязычным работникам"
	 * 
	 * пример:
	 * $text=recognize("/path/to/file/captcha.jpg","ваш_ключ_из_админки",true,"antigate.com");
	 * $text=recognize("/path/to/file/captcha.jpg","ваш_ключ_из_админки",false,"antigate.com");  //отключено комментирование
	 * $text=recognize("/path/to/file/captcha.jpg","ваш_ключ_из_админки",false,"antigate.com",1,0,0,5);  //отключено комментирование, капча состоит из двух слов, общая минимальная длина равна 5 символам
	 */
	public function recognize_antigate($filename, $apikey, $is_verbose = false, $domain = "antigate.com", $rtimeout = 5, $mtimeout = 120, $is_phrase = 0, $is_regsense = 0, $is_numeric = 0, $min_len = 0, $max_len = 0, $is_russian = 0) 
	{
		if (!file_exists($filename))
		{
			if ($is_verbose) echo "file $filename not found\n";
			return false;
		}
	    $postdata = array(
	        'method'     => 'post', 
	        'key'        => $apikey, 
	        'file'       => '@'.$filename, //полный путь к файлу
	        'phrase'	 => $is_phrase,
	        'regsense'	 => $is_regsense,
	        'numeric'	 => $is_numeric,
	        'min_len'	 => $min_len,
	        'max_len'	 => $max_len,
	    	'is_russian' => $is_russian
	    );
	    
	    $ch = curl_init();
	    curl_setopt($ch, CURLOPT_URL,            "http://$domain/in.php");
	    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	    curl_setopt($ch, CURLOPT_TIMEOUT,        60);
	    curl_setopt($ch, CURLOPT_POST,           1);
	    curl_setopt($ch, CURLOPT_POSTFIELDS,     $postdata);
	    $result = curl_exec($ch);
	    if (curl_errno($ch)) 
	    {
	    	if ($is_verbose) echo "CURL returned error: ".curl_error($ch)."\n";
	        return false;
	    }
	    curl_close($ch);
	    if (strpos($result, "ERROR")!==false)
	    {
	    	if ($is_verbose) echo "server returned error: $result\n";
	        return false;
	    }
	    else
	    {
	        $ex = explode("|", $result);
	        $captcha_id = $ex[1];
	    	if ($is_verbose) echo "captcha sent, got captcha ID $captcha_id\n";
	        $waittime = 0;
	        if ($is_verbose) echo "waiting for $rtimeout seconds\n";
	        sleep($rtimeout);
	        while(true)
	        {
	            $result = file_get_contents("http://$domain/res.php?key=$apikey&action=get&id=$captcha_id");
	            if (strpos($result, 'ERROR')!==false)
	            {
	            	if ($is_verbose) echo "server returned error: $result\n";
	                return false;
	            }
	            if ($result=="CAPCHA_NOT_READY")
	            {
	            	if ($is_verbose) echo "captcha is not ready yet\n";
	            	$waittime += $rtimeout;
	            	if ($waittime>$mtimeout) 
	            	{
	            		if ($is_verbose) echo "timelimit ($mtimeout) hit\n";
	            		break;
	            	}
	        		if ($is_verbose) echo "waiting for $rtimeout seconds\n";
	            	sleep($rtimeout);
	            }
	            else
	            {
	            	$ex = explode('|', $result);
	            	if (trim($ex[0])=='OK') return trim($ex[1]);
	            }
	        }
	        
	        return false;
	    }
	}
	
	/**
	 * Распознать текст через rucaptcha
	 */
	public function recognize_rucaptcha($filename, $apikey, $is_verbose = false, $sendhost = "rucaptcha.com", $rtimeout = 5, $mtimeout = 120, $is_phrase = 0, $is_regsense = 1, $is_numeric = 0, $min_len = 0, $max_len = 0, $language = 1)
	{
		if (!file_exists($filename))
		{
			if ($is_verbose) echo "file $filename not found\n";
			return false;
		}
	
		$fp=fopen($filename,"r");
		if ($fp!=false)
		{
			$body="";
			while (!feof($fp)) $body.=fgets($fp,1024);
			fclose($fp);
			$ext=strtolower(substr($filename,strpos($filename,".")+1));
		}
		else
		{
			if ($is_verbose) echo "could not read file $filename\n";
			return false;
		}
		 
		if ($ext=="jpg") $conttype="image/pjpeg";
		if ($ext=="gif") $conttype="image/gif";
		if ($ext=="png") $conttype="image/png";
		 
		 
		$boundary="---------FGf4Fh3fdjGQ148fdh";
		 
		$content="--$boundary\r\n";
		$content.="Content-Disposition: form-data; name=\"method\"\r\n";
		$content.="\r\n";
		$content.="post\r\n";
		$content.="--$boundary\r\n";
		$content.="Content-Disposition: form-data; name=\"key\"\r\n";
		$content.="\r\n";
		$content.="$apikey\r\n";
		$content.="--$boundary\r\n";
		$content.="Content-Disposition: form-data; name=\"phrase\"\r\n";
		$content.="\r\n";
		$content.="$is_phrase\r\n";
		$content.="--$boundary\r\n";
		$content.="Content-Disposition: form-data; name=\"regsense\"\r\n";
		$content.="\r\n";
		$content.="$is_regsense\r\n";
		$content.="--$boundary\r\n";
		$content.="Content-Disposition: form-data; name=\"numeric\"\r\n";
		$content.="\r\n";
		$content.="$is_numeric\r\n";
		$content.="--$boundary\r\n";
		$content.="Content-Disposition: form-data; name=\"min_len\"\r\n";
		$content.="\r\n";
		$content.="$min_len\r\n";
		$content.="--$boundary\r\n";
		$content.="Content-Disposition: form-data; name=\"max_len\"\r\n";
		$content.="\r\n";
		$content.="$max_len\r\n";
		$content.="--$boundary\r\n";
		$content.="Content-Disposition: form-data; name=\"language\"\r\n";
		$content.="\r\n";
		$content.="$language\r\n";
		$content.="--$boundary\r\n";
		$content.="Content-Disposition: form-data; name=\"file\"; filename=\"capcha.$ext\"\r\n";
		$content.="Content-Type: $conttype\r\n";
		$content.="\r\n";
		$content.=$body."\r\n";
		$content.="--$boundary--";
		 
		 
		$poststr="POST http://$sendhost/in.php HTTP/1.0\r\n";
		$poststr.="Content-Type: multipart/form-data; boundary=$boundary\r\n";
		$poststr.="Host: $sendhost\r\n";
		$poststr.="Content-Length: ".strlen($content)."\r\n\r\n";
		$poststr.=$content;
		 
		if ($is_verbose) echo "connecting $sendhost...";
		$fp=fsockopen($sendhost,80,$errno,$errstr,30);
		if ($fp!=false)
		{
			if ($is_verbose) echo "OK\n";
			if ($is_verbose) echo "sending request ".strlen($poststr)." bytes...";
			fputs($fp,$poststr);
			if ($is_verbose) echo "OK\n";
			if ($is_verbose) echo "getting response...";
			$resp="";
			while (!feof($fp)) $resp.=fgets($fp,1024);
			fclose($fp);
			$result=substr($resp,strpos($resp,"\r\n\r\n")+4);
			if ($is_verbose) echo "OK\n";
		}
		else
		{
			if ($is_verbose) echo "could not connect to anti-captcha\n";
			if ($is_verbose) echo "socket error: $errno ( $errstr )\n";
			return false;
		}
		 
		if (strpos($result, "ERROR")!==false or strpos($result, "<HTML>")!==false)
		{
			if ($is_verbose) echo "server returned error: $result\n";
			return false;
		}
		else
		{
			$ex = explode("|", $result);
			$captcha_id = $ex[1];
			if ($is_verbose) echo "captcha sent, got captcha ID $captcha_id\n";
			$waittime = 0;
			if ($is_verbose) echo "waiting for $rtimeout seconds\n";
			sleep($rtimeout);
			while(true)
			{
				$result = file_get_contents('http://rucaptcha.com/res.php?key='.$apikey.'&action=get&id='.$captcha_id);
				if (strpos($result, 'ERROR')!==false)
				{
					if ($is_verbose) echo "server returned error: $result\n";
					return false;
				}
				if ($result=="CAPCHA_NOT_READY")
				{
					if ($is_verbose) echo "captcha is not ready yet\n";
					$waittime += $rtimeout;
					if ($waittime>$mtimeout)
					{
						if ($is_verbose) echo "timelimit ($mtimeout) hit\n";
						break;
					}
					if ($is_verbose) echo "waiting for $rtimeout seconds\n";
					sleep($rtimeout);
				}
				else
				{
					$ex = explode('|', $result);
					if (trim($ex[0])=='OK') return trim($ex[1]);
				}
			}
			 
			return false;
		}
	}
	
}
?>