<?php
/**
 * Класс для работы с папками
 * 
 * @version 2.1.1
 */
class Dir {
	
	public $path;
	public $files;
	public $dirs;
	
	public function __construct($path=false) {
		
		if ( !$path ) return false;
		
		// если папка не существует - создаём
		if ( !file_exists($path) ) $this->create_dir($path);

		// путь гарантированно заканчивается на слэш - /
		$path = rtrim($path, '/');
		$path .= '/';            
		
		// устанавливаем параметры
		$this->path  = $path;                   // путь
		$this->files = $this->get_files_list(); // список файлов
		$this->dirs  = $this->get_dirs_list();  // список поддиректорий
		
		return true;
	}
	
	
	//*************************//
	// ПОЛУЧИТЬ ДАННЫЕ О ПАПКЕ //
	//*************************//
	public function get_files_list($not_index=true, $ext='') {
		
		$list = array();
		
		$handler = opendir($this->path);
		while ( ($file = readdir($handler)) !== false ) {
			if ($file=='.') continue;
			if ($file=='..') continue;
			
			if ( $not_index && ($file=='index.html' || $file=='index.htm' || $file=='index.php') ) continue;
			if ( $ext && get_extension($file)!=$ext ) continue;
			
			$type = filetype($this->path.$file);
			if ( $type!='dir' ) $list[] = $file;
        }
		closedir($handler);
        
		sort($list);
		
		return $list;
	}
	
	public function get_dirs_list() {
		$list = array();
		
		$handler = opendir($this->path);
		while ( ($file = readdir($handler)) !== false ) {
			if ($file=='.') continue;
			if ($file=='..') continue;
			
			$type = filetype($this->path.$file);
			if ( $type=='dir' ) $list[] = $file;
        }
        closedir($handler);
        
        sort($list);
        
		return $list;
	}
	
	public function file_exists($filename) {
		$filename = $this->correct_name($filename);
		return ( file_exists($this->path.$filename) ) ? true : false;
	}
	
	public function get_all_files($ext=false) {
		return $this->get_files_from_subdir($this->path, $ext);
	}
	
	private function get_files_from_subdir($path, $ext=false) {
		
		// преобразуем переданные расширения в массив
		if ( !$ext ) {
			$exts = array();
		} elseif ( is_array($ext) ) {
			$exts = $ext;
		} elseif ( is_string($ext) ) {
			$exts = explode(',', $ext);
		} else {
			$exts = array( $ext );
		}
		
		$result = array();
		$files_list = array();
		$dirs_list = array();
		
		// проверим, существует ли указанная папка или указан путь к файлу
		if ( !file_exists($path) ) return array();
		if ( filetype($path)=='file' ) return ( $ext && !in_array($this->get_extension($path), $exts) ) ? array() : array($path);
		
		// путь должен заканчиваться на слэш
		$path = rtrim($path, '/');
		$path .= '/';
		
		// получаем списки файлов и папок в указанной директории
		$handler = opendir($path);
		while ( ($file = readdir($handler)) !== false ) {
			if ($file=='.') continue;
			if ($file=='..') continue;
			
			$filepath = $path . $file;
			
			$type = filetype($filepath);
			if ( $type=='dir' ) $dirs_list[] = $filepath;
			if ( $type=='file' ) $files_list[] = $filepath;
		}
		closedir($handler);
		
		// получаем списки файлов из поддиректорий
		foreach ($dirs_list as $path_dir) {
			$files = $this->get_files_from_subdir($path_dir, $ext);
			$result = array_merge($result, $files);
		}
		
		// заносим в список файлы из текущей папки
		foreach ($files_list as $file_path) {
			if ( $ext && !in_array($this->get_extension($file_path), $exts) ) continue;
			$result[] = $file_path;
		}
		
		return $result;
	}
	
	
	//*********************//
	// ДЕЙСТВИЯ НАД ПАПКОЙ //
	//*********************//
	public function clear() {
		
		$files = $this->get_files_list(false);
		foreach($files as $file) unlink($this->path.$file);
		
		$dirs = $this->get_dirs_list();
		foreach($dirs as $dir) {
			$ndir = new Dir($this->path.$dir);
			$ndir->clear();
			rmdir($this->path.$dir);
		}
		
		return true;
	}
	
	public function delete() {
		$this->clear();
		rmdir($this->path);
		
		return ( file_exists($this->path) ) ? false : true;
	}
	
	public function create_file($filename, $data='') {
		if ( !$data ) $data = '';
		$filename = $this->correct_name($filename);
		file_put_contents($this->path.$filename, $data);
		return true;
	}
	
	public function copy_dir($to_path, $permissions=0777) {
		$this->rec_copy($this->path, $to_path, $permissions);
		return true;
	}
	
	private function rec_copy($from_path, $to_path, $permissions=0777) {
		$to_path = rtrim($to_path, '/');
		$to_path .= '/';
	
		$from_path = rtrim($from_path, '/');
		$from_path .= '/';
	
		if ( !file_exists($to_path) ) {
			if ($permissions)
			mkdir($to_path, $permissions);
			else
			mkdir($to_path);
		}
	
		$dir = new Dir($from_path);
	
		$files = $dir->get_files_list(false);
		$dirs = $dir->get_dirs_list();
	
		foreach ($files as $file) {
			copy($from_path.$file, $to_path.$file);
			if ($permissions) chmod($to_path.$file, $permissions);
		}
		foreach ($dirs as $dir) $this->rec_copy($from_path.$dir.'/', $to_path.$dir.'/', $permissions);
	
		return true;
	}
	
	
	//****************//
	// ПРОВЕРКА ПАПОК //
	//****************//
	public function is_empty() {
		
		$files = $this->get_files_list();
		if ( $files ) return false;
		
		$dirs = $this->get_dirs_list();
		if ( $dirs ) return false;
		
		return true;
	}
	
	
	//*************************//
	// ВСПОМОГАТЕЛЬНЫЕ ФУНКЦИИ //
	//*************************//
	private function correct_name($name) {
		$symbols = array('\\', '/', ':', '*', '?', '"', '<', '>', '|');
		foreach($symbols as $symbol) $name = str_replace($symbol, '', $name);
		return $name;
	}
	
	private function create_dir($path) {
		
		$path_arr = explode('/',$path);
		if ($path_arr[0]=='') {
			$path_arr[1] = $_SERVER['DOCUMENT_ROOT'].'/'.$path_arr[1];
			unset($path_arr[0]);
		}
		
		$path_tmp  = '';
		
		foreach($path_arr as $level) {
			if (!$level) continue;
			$path_tmp .= $level.'/';
			if ( file_exists($path_tmp) ) continue;
			mkdir($path_tmp);
		}
		
		return true;
	}
	
	private function get_extension($filename) {
		$filename = basename($filename);
		$parts = explode('.', $filename);
		if ( count($parts)==1 ) return '';
		return end($parts);
	}
}
