<?php
class Google {
	
	public $browser;
	public $parser;
	public $gate_api;
	public $use_proxy = false;
	
	public $ya_xml = '';
	public $browser_ya_xml = false;
	
	public function __construct($gate_api='') {
		global $settings;
		$this->settings = $settings;
		
		$this->browser = new Curl(true, true, false);
		//$this->browser->set_sleep(2,5);
		
		$proxy = new Proxy();
        $proxy_list = $proxy->get_proxy_list_ok();
        if ($proxy_list){
            $this->browser->use_proxy($proxy_list['list'], false, $proxy_list['auth']);
            $this->browser->shuffle_proxy();
			$this->use_proxy = true;
        }
		
		$this->parser = new Parser();
		$this->gate_api = $gate_api;
		
		$this->IDNA = new idna_convert();
		
		if ( $settings->ya_xml ) {
			$this->ya_xml = $settings->ya_xml;
			$this->browser_ya_xml = new Curl(true, false, false);
			$this->browser_ya_xml->set_sleep(1,1);
		}
	}
	
	public function get_indexed($site) {
		$site = str_replace(array('http://', 'https://', '/', ' '), '', $site);
		$query = "site:$site";
		
		$html = $this->browser->get_page('https://www.google.ru/search?q='.urlencode($query).'&oq='.urlencode($query).'&sourceid=chrome&ie=UTF-8', '', true);
		
		if ( strpos($html, 'document.getElementById(\'captcha\')')!==false ) {
			require_once LIBS . '/decaptcha/autoload.php';
			
			$sitekey = $this->parser->parse_string($html, '|data-sitekey="(.*?)"|i');
			$q = $this->parser->parse_string($html, "|name='q' value='(.*?)'|i");
			$continue = $this->parser->parse_string($html, '|name="continue" value="(.*?)"|i');
			$location = $this->parser->parse_string($html, '#Location: (.*)#i');
			
			$source = 'services\\'.$this->settings->antigate_source.'ReCaptcha';
			$captcha = new $source([
				$source::ACTION_FIELD_KEY => $this->gate_api,
				$source::ACTION_FIELD_GOOGLEKEY => $sitekey,
				$source::ACTION_FIELD_PAGEURL => trim($location),
			]);
			
			if ($captcha->recognize()) {
					$post = array(
						'g-recaptcha-response' => $captcha->getCode(),
						'q' => $q,
						'continue' => $continue,
					);
					if($this->use_proxy) $this->browser->set_channel();
					$html = $this->browser->get_page(trim($location), http_build_query($post));
			} else {
				file_put_contents(LOG . '/error_captcha.txt', date('Y-m-d H-i-s').' - '.$captcha->getError().PHP_EOL, FILE_APPEND);
			}
		}
		if($this->use_proxy) $this->browser->shuffle_proxy();
		
		$result = $this->parser->get_div_content($html, '<div id=resultStats>');
		if ( !$result ) $result = $this->parser->get_div_content($html, '<div id="resultStats">');
		if ( !$result ) $result = $this->parser->get_div_content($html, '<div class="sd" id="resultStats">');
		
		$result = strip_tags($result);
		$result = str_replace(array('&#160;', ','), '', $result);
		$result = preg_replace('|\(.*?\)|', '', $result);
		$result = preg_replace('|[^\d,]|', '', $result);
		
		return $result;
	}
	
	public function get_yandex($site) {
		if ( $this->ya_xml ) return $this->get_yandex_by_xml($site);
		
		$site = str_replace(array('http://', 'https://', '/', 'www.', ' '), '', $site);
		$query = "url:www.$site/* | url:$site/* | url:$site | url:www.$site";
		
		$html = $this->browser->get_page('https://yandex.ru/yandsearch?text='.urlencode($query));
		
		if ( strpos($html, '<title>Ой!</title>')!==false ) {
			require_once LIBS . '/decaptcha/autoload.php';
			
			for($i=0; $i <= 5; $i++) {
				$key = $this->parser->parse_string($html, '|name="key" value="(.*?)"|i');
				$retpath = $this->parser->parse_string($html, '|name="retpath" value="(.*?)"|i');
				
				$img = $this->parser->parse_string($html, '|<img src="([^"]+)" class="b-captcha__image">|');
				if ( !$img ) $img = $this->parser->parse_string($html, '|<img class="image form__captcha"[^>]*src="(.*?)"|i');
				
				$img = $this->browser->get_page($img);
				
				$fname = LOG . '/' . get_uniq_id() . '.jpg';
				file_put_contents($fname, $img);
				
				$source = 'services\\'.$this->settings->antigate_source;
				$captcha = new $source([
					$source::ACTION_FIELD_KEY => $this->gate_api,
					$source::ACTION_FIELD_FILE => $fname,
					$source::ACTION_FIELD_PHRASE => 1,
					$source::ACTION_FIELD_REGSENSE => 0,
				]);
				if ($captcha->recognize($fname)) {
					$response = $captcha->getCode();
				} else {
					file_put_contents(LOG . '/error_captcha.txt', date('Y-m-d H-i-s').' - '.$captcha->getError().PHP_EOL, FILE_APPEND);
					$captcha->notTrue();
					unlink($fname);
					continue;
				}
				unlink($fname);
				
				if($this->use_proxy) $this->browser->set_channel();
				$html = $this->browser->get_page('http://yandex.ru/checkcaptcha?rep='.urlencode($response).'&key='.urlencode($key).'&retpath='.urlencode($retpath));
				
				if ( strpos($html, '<title>Ой!</title>')!==false ) {
					$captcha->notTrue();
					if($this->use_proxy) $this->browser->shuffle_proxy();
					continue;
				} else {
					break;
				}
			}
		}
		if($this->use_proxy) $this->browser->shuffle_proxy();
	
		$result = $this->parser->parse_string($html, '|Яндекс: наш(.*?)ответ|i');
		if ( !$result ) $result = $this->parser->parse_string($html, '|Яндекс: наш(.*?)результ|i');
		if ( !$result ) $result = $this->parser->get_tags_by_class($html, 'strong', 'b-head-logo__text', true);
		if ( !$result ) $result = $this->parser->get_tags_by_class($html, 'span', 'input__found-text', true);
		
		$result = str_replace(array('тыс', 'млн'), array('000', '000000'), $result);
		
		$ya['index'] = preg_replace('|[^\d]|', '', $result);
		if ( !$ya['index'] ) {
			$ya['index'] = 0;
		}
		if($this->settings->sba_api){
			$ya['virus'] = $this->browser->get_page('https://sba.yandex.net/lookup?client=api&apikey=' . $this->settings->sba_api . '&pver=3.5&url=http://' . $site);
			if( strlen($ya['virus']) > 30 ) $ya['virus'] = '';
		}
		
		$ya['adalt'] = $this->browser->get_page('https://sba.yandex.net/cp?client=api&pver=4.0&url=http://' . $site);
		if( strlen($ya['adalt']) > 30 ) $ya['adalt'] = '';
		
		if( !$ya['virus'] && ( stripos($html, 'Будьте осторожны при скачивании файлов на этом сайте') || stripos($html, 'Сайт может угрожать безопасности вашего компьютера или мобильного устройства') ) ){
			$ya['virus'] = 'index warn';
		}
		
		return $ya;
	}
	
	public function get_yandex_by_xml($site) {
		$site = str_replace(array('http://', 'https://', '/', 'www.', ' '), '', $site);
		$query = "url:www.$site/* | url:$site/* | url:$site | url:www.$site";
		$query = urlencode($query);

		$xml = $this->browser_ya_xml->get_page($this->ya_xml . "&query={$query}");
		
		$response = new SimpleXMLElement($xml);
		$response = $response->response;
		
		if ( isset($response->found[2]) ) $ya['index'] = (int)$response->found[2];
		if ( isset($response->found[1]) ) $ya['index'] = (int)$response->found[1];
		if ( isset($response->found[0]) ) $ya['index'] = (int)$response->found[0];
		
		if($this->settings->sba_api){
			$ya['virus'] = $this->browser->get_page('https://sba.yandex.net/lookup?client=api&apikey=' . $this->settings->sba_api . '&pver=3.5&url=http://' . $site);
			if( strlen($ya['virus']) > 30 ) $ya['virus'] = '';
		}
		
		$ya['adalt'] = $this->browser->get_page('https://sba.yandex.net/cp?client=api&pver=4.0&url=http://' . $site);
		if( strlen($ya['adalt']) > 30 ) $ya['adalt'] = '';
		
		return $ya;
	}
	
	public function get_yandex_video($site) {
		$site = str_replace(array('http://', 'https://', '/', ' '), '', $site);
		$query = "url:www.$site/* | url:$site/* | url:$site | url:www.$site";
		
		$html = $this->browser->get_page('https://yandex.ru/video/search?text='.urlencode($query));
		
		if ( strpos($html, '<title>Ой!</title>')!==false ) {
			require_once LIBS . '/decaptcha/autoload.php';
			
			for($i=0; $i <= 5; $i++) {
				$key = $this->parser->parse_string($html, '|name="key" value="(.*?)"|i');
				$retpath = $this->parser->parse_string($html, '|name="retpath" value="(.*?)"|i');
				
				$img = $this->parser->parse_string($html, '|<img src="([^"]+)" class="b-captcha__image">|');
				if ( !$img ) $img = $this->parser->parse_string($html, '|<img class="image form__captcha"[^>]*src="(.*?)"|i');
				
				$img = $this->browser->get_page($img);
				
				$fname = LOG . '/' . get_uniq_id() . '.jpg';
				file_put_contents($fname, $img);
				
				$source = 'services\\'.$this->settings->antigate_source;
				$captcha = new $source([
					$source::ACTION_FIELD_KEY => $this->gate_api,
					$source::ACTION_FIELD_FILE => $fname,
					$source::ACTION_FIELD_PHRASE => 1,
					$source::ACTION_FIELD_REGSENSE => 0,
				]);
				if ($captcha->recognize($fname)) {
					$response = $captcha->getCode();
				} else {
					file_put_contents(LOG . '/error_captcha.txt', date('Y-m-d H-i-s').' - '.$captcha->getError().PHP_EOL, FILE_APPEND);
					$captcha->notTrue();
					unlink($fname);
					continue;
				}
				unlink($fname);
				
				if($this->use_proxy) $this->browser->set_channel();
				$html = $this->browser->get_page('http://yandex.ru/checkcaptcha?rep='.urlencode($response).'&key='.urlencode($key).'&retpath='.urlencode($retpath));
				
				if ( strpos($html, '<title>Ой!</title>')!==false ) {
					$captcha->notTrue();
					if($this->use_proxy) $this->browser->shuffle_proxy();
					continue;
				} else {
					break;
				}
			}
		}
		if($this->use_proxy) $this->browser->shuffle_proxy();
		
		$html = htmlspecialchars_decode($html);
		
		$result = $this->parser->parse_string($html, '|"foundNum":"([\d\s]+)"|');
		if ( !$result ) $result = $this->parser->parse_string($html, '|mdash;(.*?)видео"|i');
		if ( !$result ) $result = $this->parser->parse_string($html, '|\("\—(.*?). видео"\)|i');
		if ( !$result ) $result = $this->parser->parse_string($html, '|foundValue\\\u0022:\\\u0022—(.*?)видео|i');
		if ( !$result ) $result = $this->parser->parse_string($html, '|<div class="input__found">(.*?)</div>|i');
		
		$result = str_replace(array('тыс', 'млн'), array('000', '000000'), $result);
		$result = preg_replace('|[^\d]|', '', $result);
		
		if ( !$result ) $result = 0;
		return $result;
	}
	
	public function get_mail($site) {
		$site = str_replace(array('http://', 'https://', '/', ' '), '', $site);
		$html = $this->browser->get_page("http://go.mail.ru/search?site=$site&advs=1&q=$site");
		
		if ( strpos($html, 'Доступ к поисковой системе с вашего компьютера временно ограничен')!==false || strpos($html, 'Мы ограничили ваш доступ к Поиску')!==false ){
			require_once LIBS . '/decaptcha/autoload.php';
			
			for($i=0; $i <= 5; $i++) {
				$sqid = $this->parser->parse_string($html, '|name="sqid" value="(.*?)"|i');
				$img = 'http://go.mail.ru/ar_captcha?id='.$sqid;
				
				$img = $this->browser->get_page($img);
				
				$fname = LOG . '/' . get_uniq_id() . '.jpg';
				file_put_contents($fname, $img);
				
				$source = 'services\\'.$this->settings->antigate_source;
				$captcha = new $source([
					$source::ACTION_FIELD_KEY => $this->gate_api,
					$source::ACTION_FIELD_FILE => $fname,
					$source::ACTION_FIELD_REGSENSE => 0,
				]);
				if ($captcha->recognize($fname)) {
					$response = $captcha->getCode();
				} else {
					file_put_contents(LOG . '/error_captcha.txt', date('Y-m-d H-i-s').' - '.$captcha->getError().PHP_EOL, FILE_APPEND);
					$captcha->notTrue();
					unlink($fname);
					continue;
				}
				
				unlink($fname);
				
				if($this->use_proxy) $this->browser->set_channel();
				$html = $this->browser->get_page('http://go.mail.ru/ar?sqid='.urlencode($sqid).'&SequreWord='.$response.'&q='.urlencode($site).'&back='.urlencode('http://go.mail.ru/search?site='.$site.'&advs=1&q='.$site));
			
				if ( strpos($html, 'Доступ к поисковой системе с вашего компьютера временно ограничен')!==false || strpos($html, 'Мы ограничили ваш доступ к Поиску')!==false ) {
					$captcha->notTrue();
					if($this->use_proxy) $this->browser->shuffle_proxy();
					continue;
				} else {
					break;
				}
			}
		}
		if($this->use_proxy) $this->browser->shuffle_proxy();
		
		$result = $this->parser->parse_string($html, '|foundCount: ([\d\s]+),|');
		
		if ( !$result ) $result = 0;
		return $result;
	}
	
	public function get_bing($site) {
		$site = str_replace(array('http://', 'https://', '/', ' '), '', $site);
		$html = $this->get_bing_page('site%3A'.$site);
		
		$result = $this->parser->parse_string($html, '|<span class="sb_count">(.*?)</span>|');
		$result = str_replace('&#160;', '', $result);
		$result = preg_replace('|[^\d]|', '', $result);
		$result = intval($result);
		
		return $result;
	}
	
	public function get_cy($site) {
		$site = str_replace(array('http://', 'https://', '/', ' '), '', $site);
		$iks = new YandexIks();
		
		return $iks->getValueFromImage($site);
	}
	
	protected function get_bing_page($query) {
		$header[] = 'Host: www.bing.com';
		$header[] = 'User-Agent: Mozilla/5.0 (Windows NT 6.1; rv:32.0) Gecko/20100101 Firefox/32.0';
		$header[] = 'Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8';
		$header[] = 'Accept-Language: ru-RU,ru;q=0.8,en-US;q=0.5,en;q=0.3';
		$header[] = 'Accept-Encoding: none';
		$header[] = 'Referer: http://www.bing.com/';
		$header[] = 'Cookie: MUID=1C5788339A2B642519738E3A9E2B629B; OrigMUID=3E7DDD08F2E3674D01CBDB08F3C2670B%2cf9419990841b4701bb916b7d8f01ac64; SRCHD=MS=3265584&D=3265580&AF=NOFORM; SRCHUSR=AUTOREDIR=0&GEOVAR=&DOB=20140317; SRCHUID=V=2&GUID=BCBA76543C0143AB910A345453237B6C; _EDGE_V=1; MUIDB=1C5788339A2B642519738E3A9E2B629B; SRCHHPGUSR=CW=1903&CH=694; _HOP=; _SS=SID=BBFB0DC996A14315A5874F0F28755C15&bIm=433567; SCRHDN=ASD=0&DURL=#; WLS=TS=63548208587';
		$header[] = 'Connection: keep-alive';

		$url='http://www.bing.com/search?q='.$query.'&go=%D0%9E%D1%82%D0%BF%D1%80%D0%B0%D0%B2%D0%B8%D1%82%D1%8C+%D0%B7%D0%B0%D0%BF%D1%80%D0%BE%D1%81&qs=bs&form=QBRE';
		
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL,$url);
		curl_setopt($ch, CURLOPT_HEADER, 1);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
		curl_setopt($ch, CURLOPT_TIMEOUT, 30);
		curl_setopt ($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
		curl_setopt ( $ch , CURLOPT_HTTPHEADER , $header);
		$result = curl_exec($ch);
		curl_close($ch);
		
		return $result;
	}
	
	public function get_pr($url) {
		return 0;
	}
	
	// яндекс.метрика
	public function get_yam_counters() {
		$token = $this->settings->yam_token;
		
		if ( !$token ) return array();
		
		$counters = file_get_contents("https://api-metrika.yandex.ru/management/v1/counter/{$counter_id}?oauth_token={$token}");
		$counters = json_decode($counters, true);
		
		$result = array();
		
		foreach ($counters['counters'] as $row)
			$result[ $row['site'] ] = $row;
		
		return $result;
	}
	
	public function get_yam_stat($counter_id) {
		$token = $this->settings->yam_token;
		
		$stat = array(
			'today_visitors' => 0,
			'today_visitors_diff' => 0,
			
			'today_hits' => 0,
			'today_hits_diff' => 0,
				
			'yesterday_visitors' => 0,
			'yesterday_hits' => 0,
		);
		
		if ( !$token ) return $stat;
		
		$query = array(
			'date1' => date('Ymd', time()-86400),
			'date2' => date('Ymd'),
			'ids' => $counter_id,
			'metrics' => 'ym:s:visits,ym:s:pageviews',
			'oauth_token' => $token,
			'group' => 'day'
		);
        $data = file_get_contents('https://api-metrika.yandex.ru/stat/v1/data/bytime?'.http_build_query($query));
		$data = json_decode($data);
		
		$stat = array(
			'today_visitors' => $data->data[0]->metrics[0][1],
			'today_visitors_diff' => 0,
			
			'today_hits' => $data->data[0]->metrics[1][1],
			'today_hits_diff' => 0,
				
			'yesterday_visitors' => $data->data[0]->metrics[0][0],
			'yesterday_hits' => $data->data[0]->metrics[1][0],
		);
		
		return $stat;
	}
	
	public function get_yam_ps_stat($counter_id) {
		$token = $this->settings->yam_token;
		
		if ( !$token ) return array();
		
		$query = array(
			'date1' => date('Ymd', time()-86400),
			'date2' => date('Ymd'),
			'ids' => $counter_id,
			'dimensions' => 'ym:s:<attribution>SearchEngineRoot',
			'metrics' => 'ym:s:visits',
			'oauth_token' => $token,
			'group' => 'day'
		);
		
		$data = file_get_contents('https://api-metrika.yandex.ru/stat/v1/data/bytime?'.http_build_query($query));
		$data = json_decode($data);
		// Яндекс
		// Google
		// Mail.ru
		// Bing
		foreach( $data->data as $row ){
			$stat[] = array(
				'ps' => $row->dimensions[0]->name,
				'today' => $row->metrics[0][1],
				'yesterday' => $row->metrics[0][0],
			);
		}
		
		return $stat;
	}
	
	public function get_yam_wap_stat($counter_id) {
		$token = $this->settings->yam_token;
		
		$stat = array(
			'today' => 0,
			'yesterday' => 0
		);
		
		if ( !$token ) return $stat;
		
		$query = array(
			'date1' => date('Ymd', time()-86400),
			'date2' => date('Ymd'),
			'ids' => $counter_id,
			'metrics' => 'ym:s:visits',
			'filters' => "ym:s:isMobile=='Yes'",
			'oauth_token' => $token,
			'group' => 'day'
		);
		$data = file_get_contents('https://api-metrika.yandex.ru/stat/v1/data/bytime?'.http_build_query($query));
		$data = json_decode($data);
		
		$stat = array(
			'today' => ($data->data[0]->metrics[0][1]) ? $data->data[0]->metrics[0][1] : '0',
			'yesterday' => ($data->data[0]->metrics[0][0]) ? $data->data[0]->metrics[0][0] : '0'
		);
		
		return $stat;
	}
	
	public function get_yam_month_stat($counter_id){
		$token = $this->settings->yam_token;
		
		$query = array(
			'date1' => date('Ymd', time()-31536000),
			'date2' => date('Ymd'),
			'ids' => $counter_id,
			'metrics' => 'ym:s:visits,ym:s:pageviews',
			'oauth_token' => $token,
			'group' => 'month'
		);
		
		$data = file_get_contents('https://api-metrika.yandex.ru/stat/v1/data/bytime?'.http_build_query($query));
		$data = json_decode($data);
		
		foreach($data->data[0]->metrics[0] as $key => $val){
			$stat[] = array(
				'date' => date('M y', strtotime($data->time_intervals[$key][0])),
				'hits' => $data->data[0]->metrics[1][$key],
				'visitors' => $data->data[0]->metrics[0][$key],
			);
		}
		
		return json_encode($stat);
	}
	
	public function get_yam_key($counter_id, $start, $end) {
		$token = $this->settings->yam_token;
		
		$query = array(
			'date1' => date('Ymd', strtotime($start)),
			'date2' => date('Ymd', strtotime($end)),
			'ids' => $counter_id,
			'dimensions' => 'ym:s:<attribution>SearchPhrase',
			'metrics' => 'ym:s:visits',
			'oauth_token' => $token,
			'group' => 'day'
		);
		$data = file_get_contents('https://api-metrika.yandex.ru/stat/v1/data/bytime?'.http_build_query($query));
		$data = json_decode($data);
		
		return array();
	}
	
	
	// google analytics
	public function get_ga_stat($ga_id) {
		$token = $this->settings->ga_token;
		
		$stat = array(
			'today_visitors' => 0,
			'today_visitors_diff' => 0,
				
			'today_hits' => 0,
			'today_hits_diff' => 0,
	
			'yesterday_visitors' => 0,
			'yesterday_hits' => 0,
		);
		
		if ( !$token ) return $stat;
		
		$today = date('Y-m-d');
		$yesterday = date('Y-m-d', time()-86400);
		
		$client = new Google_Client();
		 
		$client->setClientId( $this->settings->ga_id );
		$client->setClientSecret( $this->settings->ga_secret );
		$client->setRedirectUri( 'http://' . $this->settings->host . '/ajax/ga.php' );
		$client->setDeveloperKey( $this->settings->ga_api_key );
		$client->setScopes( array('https://www.googleapis.com/auth/analytics.readonly', 'https://www.googleapis.com/auth/analytics') );
		$client->setAccessType('offline');
		$client->setApprovalPrompt('force');
		
		$oauth2 = new Google_Service_Oauth2($client);
		
		$client->setAccessToken( $this->settings->ga_token );
		
		$expired = $client->isAccessTokenExpired();
		if ( $expired ) {
			$client->refreshToken( $this->settings->ga_refresh_token );
			$this->settings->save_option('ga_token', $client->getAccessToken());
		}
		$analytics = new Google_Service_Analytics($client);
		
		$results = $analytics->data_ga->get( "ga:{$ga_id}", $today, $today, 'ga:sessions,ga:pageviews' );
		$rows = $results->rows;
		$stat['today_visitors'] = isset($rows[0][0]) ? $rows[0][0] : 0;
		$stat['today_hits'] = isset($rows[0][1]) ? $rows[0][1] : 0;
		
		$results = $analytics->data_ga->get( "ga:{$ga_id}", $yesterday, $yesterday, 'ga:sessions,ga:pageviews' );
		$rows = $results->rows;
		$stat['yesterday_visitors'] = isset($rows[0][0]) ? $rows[0][0] : 0;
		$stat['yesterday_hits'] = isset($rows[0][1]) ? $rows[0][1] : 0;
		
		return $stat;
	}
	
	public function get_ga_wap_stat($ga_id) {
		$token = $this->settings->ga_token;
		
		$stat = array(
			'today' => 0,
			'yesterday' => 0
		);
		
		if ( !$token ) return $stat;
		
		$today = date('Y-m-d');
		$yesterday = date('Y-m-d', time()-86400);
		
		$client = new Google_Client();
			
		$client->setClientId( $this->settings->ga_id );
		$client->setClientSecret( $this->settings->ga_secret );
		$client->setRedirectUri( 'http://' . $this->settings->host . '/ajax/ga.php' );
		$client->setDeveloperKey( $this->settings->ga_api_key );
		$client->setScopes( array('https://www.googleapis.com/auth/analytics.readonly', 'https://www.googleapis.com/auth/analytics') );
		$client->setAccessType('offline');
		
		$oauth2 = new Google_Service_Oauth2($client);
		
		$client->setAccessToken( $this->settings->ga_token );
		
		$expired = $client->isAccessTokenExpired();
		if ( $expired ) {
			$client->refreshToken( $this->settings->ga_refresh_token );
			$this->settings->save_option('ga_token', $client->getAccessToken());
		}
		
		$analytics = new Google_Service_Analytics($client);
		
		$results = $analytics->data_ga->get( "ga:{$ga_id}", $today, $today, 'ga:sessions,ga:pageviews', array('dimensions' => 'ga:isMobile,ga:isTablet') );
		$rows = $results->rows;
		if ( !$rows ) $rows = array();
		
		foreach ($rows as $row)
			$stat['today'] += ($row[0] == 'Yes' || $row[1] == 'Yes') ? $row[2] : 0;
		
		$results = $analytics->data_ga->get( "ga:{$ga_id}", $yesterday, $yesterday, 'ga:sessions,ga:pageviews', array('dimensions' => 'ga:isMobile,ga:isTablet') );
		$rows = $results->rows;
		
		if ( !$rows ) $rows = array();
		foreach ($rows as $row)
			$stat['yesterday'] = ($row[0] == 'Yes') ? $row[2] : 0;
		
		return $stat;
	}
	
	public function get_ga_ps_stat($ga_id) {
		$token = $this->settings->ga_token;
		
		$stat = $arr = $res = $result = $results = array();
		
		if ( !$token ) return $stat;
		
		$today = date('Y-m-d');
		$yesterday = date('Y-m-d', time()-86400);
		
		$client = new Google_Client();
		
		$client->setClientId( $this->settings->ga_id );
		$client->setClientSecret( $this->settings->ga_secret );
		$client->setRedirectUri( 'http://' . $this->settings->host . '/ajax/ga.php' );
		$client->setDeveloperKey( $this->settings->ga_api_key );
		$client->setScopes( array('https://www.googleapis.com/auth/analytics.readonly', 'https://www.googleapis.com/auth/analytics') );
		$client->setAccessType('offline');
		
		$oauth2 = new Google_Service_Oauth2($client);
		
		$client->setAccessToken( $this->settings->ga_token );
		
		$expired = $client->isAccessTokenExpired();
		if ( $expired ) {
			$client->refreshToken( $this->settings->ga_refresh_token );
			$this->settings->save_option('ga_token', $client->getAccessToken());
		}
		
		$analytics = new Google_Service_Analytics($client);
		
		$results = $analytics->data_ga->get( "ga:{$ga_id}", $yesterday, $today, 'ga:organicSearches', array('dimensions' => 'ga:source,ga:date') );
		$rows = $results->rows;
		
		$rows_today = date('Ymd');
		$rows_yesterday = date('Ymd', time()-86400);
		
		if ( !$rows ) $rows = array();
		
		foreach( $rows as $k => $v )
			$res[] = array('ps' => $v[0], 'name' => array($v[1] => $v[2]));
		
		foreach ($res as $l)
		  $result[$l['ps']][] = $l['name'];
		
		foreach ($result as $key => $val){
			$arr['ps'] = $key;
			$arr['today'] = 0;
			$arr['yesterday'] = 0;
			
			foreach($val as $ke => $va){
				foreach($va as $k => $v){
					if($k == $rows_today) $arr['today'] = $v;
					if($k == $rows_yesterday) $arr['yesterday'] = $v;
				}
			}
			// Яндекс
			// Google
			// Mail.ru
			// Bing
			$arr['ps'] = str_replace( array("yandex","google","go.mail.ru","bing"), array("Яндекс","Google","Mail.ru","Bing"), $arr['ps'] );
			
			$stat[] = $arr;
		}
  
		return $stat;
	}
		
	
	public function get_ga_month_stat($ga_id){
		$token = $this->settings->ga_token;
		
		$stat = array();
		
		if ( !$token ) return $stat;
		
		$today = date('Y-m-d');
		$yesterday = date('Y-m-d', time()-31536000);
		
		$client = new Google_Client();
		
		$client->setClientId( $this->settings->ga_id );
		$client->setClientSecret( $this->settings->ga_secret );
		$client->setRedirectUri( 'http://' . $this->settings->host . '/ajax/ga.php' );
		$client->setDeveloperKey( $this->settings->ga_api_key );
		$client->setScopes( array('https://www.googleapis.com/auth/analytics.readonly', 'https://www.googleapis.com/auth/analytics') );
		$client->setAccessType('offline');
		$client->setApprovalPrompt('force');
		
		$oauth2 = new Google_Service_Oauth2($client);
		
		$client->setAccessToken( $this->settings->ga_token );
		
		$expired = $client->isAccessTokenExpired();
		if ( $expired ) {
			$client->refreshToken( $this->settings->ga_refresh_token );
			$this->settings->save_option('ga_token', $client->getAccessToken());
		}
		$analytics = new Google_Service_Analytics($client);
		
		$results = $analytics->data_ga->get( "ga:{$ga_id}", $yesterday, $today, 'ga:sessions,ga:pageviews', array('dimensions' => 'ga:yearMonth'));
		
		foreach($results->rows as $month){
			$stat[] = array(
				'date' => date('M y', strtotime($month[0].'01')),
				'hits' => $month[2],
				'visitors' => $month[1]
			);
		}
		
		return json_encode($stat);
	}
	
	public function get_ga_key($ga_id, $start, $end) {
		$token = $this->settings->ga_token;
		
		if ( !$token ) return false;
		
		$client = new Google_Client();
		 
		$client->setClientId( $this->settings->ga_id );
		$client->setClientSecret( $this->settings->ga_secret );
		$client->setRedirectUri( 'http://' . $this->settings->host . '/ajax/ga.php' );
		$client->setDeveloperKey( $this->settings->ga_api_key );
		$client->setScopes( array('https://www.googleapis.com/auth/analytics.readonly', 'https://www.googleapis.com/auth/analytics') );
		$client->setAccessType('offline');
		$client->setApprovalPrompt('force');
		
		$oauth2 = new Google_Service_Oauth2($client);
		
		$client->setAccessToken( $this->settings->ga_token );
		
		$expired = $client->isAccessTokenExpired();
		if ( $expired ) {
			$client->refreshToken( $this->settings->ga_refresh_token );
			$this->settings->save_option('ga_token', $client->getAccessToken());
		}
		$analytics = new Google_Service_Analytics($client);
		
		# https://developers.google.com/analytics/devguides/reporting/core/dimsmets
		$params =  array(
						'dimensions'    => 'ga:keyword',
						'max-results'   => '10000',
						'start-index'   => '1',	
						);
		$results = $analytics->data_ga->get( "ga:{$ga_id}", $start, $end, 'ga:organicSearches', $params );
		foreach( $results->rows as $row )
			if($row[0] != '(not set)' && $row[0] != '(not provided)') $keys[] = $row[0];
		
		$total = ceil($results->totalResults / 10000);
		if( $total > 1 ){
			for($i=1; $i <= ($total-1); $i++) {
				$params['start-index'] = $i.'0000';
				$results = $analytics->data_ga->get( "ga:{$ga_id}", $start, $end, 'ga:organicSearches', $params );
				foreach( $results->rows as $row )
					if($row[0] != '(not set)' && $row[0] != '(not provided)') $keys[] = $row[0];
			}
		}
		
		return $keys;
	}
	
	public function get_ip($url, $ip='n/a'){
		$url = str_replace(array('http://', 'https://', '/', ' '), '', $url);
		$ips = @gethostbynamel($url);
		if($ips) foreach($ips as $ip);
		return $ip;
	}
	
	public function get_domain_info($url){
		$sheme = strpos($url, 'https')!==false ? 'https' : 'http';
		$url = str_replace(array('http://', 'https://', '/', ' '), '', $url);
		
		$mtime = microtime();
		$mtime = explode(" ",$mtime);
		$mtime = $mtime[1] + $mtime[0];
		$tstart = $mtime;
		
		$html = @file_get_contents("$sheme://$url");
		
		$mtime = microtime();
		$mtime = explode(" ",$mtime);
		$mtime = $mtime[1] + $mtime[0];
		$tend = $mtime;
		$totaltime = ($tend - $tstart);
		$info['time'] = round($totaltime,0);
		
		$string = $this->parser->parse_string($html, '|<title>(.*?)</title>|si');
		$charset = detect_encoding($string);
		if ( $charset == 'windows-1251' ) $string = ic_wu($string);
		$info['title'] = $string;
		
		$info['header'] = '0';
		if(@$http_response_header){
			foreach($http_response_header as $header){
				if ('HTTP' === substr($header, 0, 4)) 
					$info['header'] = $this->parser->parse_string($header, '!\s(.*?)\s!si');
			}
		}
		
		return $info;
	}
	
	public function get_ssl_info($url){
		$url = str_replace(array('http://', 'https://', '/', ' '), '', $url);
		$url = $this->IDNA->encode($url);
		
		$get = stream_context_create(array("ssl" => array("capture_peer_cert" => TRUE)));
		$read = @stream_socket_client("ssl://".$url.":443", $errno, $errstr, 30, STREAM_CLIENT_CONNECT, $get);
		$cert = @stream_context_get_params($read);
		if(!$cert) return array('valid_to' => '');
		$certinfo = openssl_x509_parse($cert['options']['ssl']['peer_certificate']);
		
		return array('valid_to' => date("Y-m-d", $certinfo['validTo_time_t']));
	}
	
	public function get_whois($url){
		require_once LIBS . '/whois/autoload.php';
		
		$url = str_replace(array('http://', 'https://', '/', ' '), '', $url);
		$url = $this->IDNA->encode($url);
		
		$info = Iodev\Whois\Whois::create()->loadDomainInfo($url);
		
		if($info) return array(
					'whois_created' => date("Y-m-d", $info->getCreationDate()),
					'whois_paid' => date("Y-m-d", $info->getExpirationDate()),
					'reg' => $info->getRegistrar()
				);
		
		return array(
					'whois_created' => '',
					'whois_paid' => '',
					'reg' => ''
				);
	}
	
	public function get_promosite(){
		$xml = $this->browser->get_page("http://api.promosite.ru/public/yabar.xml");
		$response = new SimpleXMLElement($xml);
		$array = (array) $response->mainbutton->tooltip;
		if(!$array[0]) $array[0] = '';
		
		return $array[0];
	}
	
	public function get_antizapret($site, $more=false) {
		$site = str_replace(array('http://', 'https://', '/', 'www.', ' '), '', $site);
		
		$json = $this->browser->get_page("https://api.antizapret.info/get.php?item=$site&type=json");
		$data = json_decode($json);
		
		if($more) return $data;
		if($data->register) return 'Фильтр Роскомнадзора';
		
		return false;
	}
}