<?php
/**
 * Драйвер для работы с сервисом статистики liveinternet.ru
 * 
 * @version  1.3.0
 */
class LiRu {
	
	private $browser;
	private $parser;
	private $use_proxy = false;
	
	private $domain;
	private $password;
	
	public function __construct($domain, $password) {
		$this->IDNA = new idna_convert();
		$this->parser = new Parser();
		
		$this->browser = new Curl();
		//$this->browser->set_sleep(1,3);
		
		$proxy = new Proxy();
        $proxy_list = $proxy->get_proxy_list_ok();
        if ($proxy_list){
            $this->browser->use_proxy($proxy_list['list'], false, $proxy_list['auth']);
            $this->browser->shuffle_proxy();
			$this->use_proxy = true;
        }
		
		$domain = str_replace(array('http://', '/', ' '), '', $domain);
		$this->domain = urlencode($this->IDNA->decode($domain));
		$this->password = $password;
		
		if ( $password ) {
			$html = $this->browser->get_page('https://www.liveinternet.ru/stat/', "url={$domain}&password={$password}", true, true);
		}
	}
	
	public function get_queries($start_date, $end_date) {
		$keys = array();
		$count_days = floor ( ( strtotime($end_date) - strtotime($start_date) ) / 86400 );
		
		// Проходимся по всем дням, собираем кейворды
		for ($i=0; $i<=$count_days; $i++) {
			$date = date("Y-m-d",strtotime("+$i days",strtotime($start_date)));
			$keys_d = $this->get_queries_day($date);
			$keys = array_merge($keys, $keys_d);
		}
		if($this->use_proxy) $this->browser->shuffle_proxy();
		
		return $keys;
	}
	
	public function get_queries_day($date) {
		$keys = array();
		$html = $this->browser->get_page( "https://www.liveinternet.ru/stat/{$this->domain}/queries.html?date={$date}&per_page=100" );
		preg_match_all('/gosearchword.*>(.*)<\/a>/Usi', $html, $res, PREG_SET_ORDER);
			foreach($res as $k)
				$keys[] = $k[1];
		
		if($this->use_proxy) $this->browser->set_channel();
		
		$page = 1;
		while (true) {
			$page += 1;
			$html = $this->browser->get_page("https://www.liveinternet.ru/stat/{$this->domain}/queries.html?date={$date};page={$page}");
			preg_match_all('/gosearchword.*>(.*)<\/a>/Usi', $html, $res, PREG_SET_ORDER);
			foreach($res as $k)
				$keys_p[] = $k[1];
			
			$end = array_diff($keys_p, $keys);
			if(empty($end)) break;
			
			$keys = array_merge($keys, $keys_p);
		}
		return $keys;
	}
	
	public function get_stat() {
		$url = "https://www.liveinternet.ru/stat/{$this->domain}/index.html";
		
		$html = $this->browser->get_page($url);
		
		$html = str_replace(array("\r\n","\r","\n"), '', $html);
		$table = $this->parser->get_table($html, '<table cellpadding=3 cellspacing=0 border=0 bgcolor=#e8e8e8 width=520>');
		
		$today_visitors = $this->parser->parse_string($table['rows'][3]['cells'][2], '|<td>([\d,]+)<br/>|');
		$today_visitors_diff = $this->parser->parse_string($table['rows'][3]['cells'][2], '|<font[^>]*>(.*?)</font>|');
		
		$today_hits = $this->parser->parse_string($table['rows'][1]['cells'][2], '|<td>([\d,]+)<br/>|');
		$today_hits_diff = $this->parser->parse_string($table['rows'][1]['cells'][2], '|<font[^>]*>(.*?)</font>|');
		
		$yesterday_visitors = $this->parser->parse_string($table['rows'][3]['cells'][3], '|<td>([\d,]+)</td>|');
		$yesterday_hits = $this->parser->parse_string($table['rows'][1]['cells'][3], '|<td>([\d,]+)</td>|');
		
		$stat = array(
			'today_visitors' => ($today_visitors) ? $today_visitors : '0',
			'today_visitors_diff' => ($today_visitors_diff) ? $today_visitors_diff : '0',
			
			'today_hits' => ($today_hits) ? $today_hits : '0',
			'today_hits_diff' => ($today_hits_diff) ? $today_hits_diff : '0',
			
			'yesterday_visitors' => ($yesterday_visitors) ? $yesterday_visitors : '0',
			'yesterday_hits' => ($yesterday_hits) ? $yesterday_hits : '0',
		);
		
		foreach ($stat as $k => $v) $stat[$k] = str_replace(',', '', $v);
		
		if($this->use_proxy) $this->browser->shuffle_proxy();
		
		return $stat;
	}
	
	public function get_wap_stat() {
		$url = "https://www.liveinternet.ru/stat/{$this->domain}/oses.html?per_page=100";
		$html = $this->browser->get_page($url);
		$html = str_replace(array("\r\n","\r","\n"), '', $html);
		
		$table = $this->parser->get_table($html, '<table cellpadding=3 cellspacing=0 border=0 bgcolor=#e8e8e8 width=520>');
		
		if ( !$table ) {
			return array(
				'today' => 0,
				'yesterday' => 0
			);
		}
		
		$today = 0;
		$yesterday = 0;
		
		foreach ($table['rows'] as $row) {
			$cells = $row['cells'];
			if ( count($cells) < 2 ) continue;
			
			if ( stripos($cells[1], 'Android')!== false || stripos($cells[1], 'iOS')!== false || stripos($cells[1], 'SymbianOS')!== false || stripos($cells[1], 'Windows Phone')!== false ) {
				$v = $cells[2];
				$v = $this->parser->clear_text($v);
				$v = str_replace(',', '', $v);
				$v = intval($v);
				
				$today += $v;
				
				$v = $cells[4];
				$v = $this->parser->clear_text($v);
				$v = str_replace(',', '', $v);
				$v = intval($v);
				
				$yesterday += $v;
			}
		}
		
		$stat = array(
			'today' => $today,
			'yesterday' => $yesterday
		);
		
		if($this->use_proxy) $this->browser->shuffle_proxy();
		
		return $stat;
	}
	
	public function get_ps_stat() {
		$stat = array();
		$url = "https://www.liveinternet.ru/stat/{$this->domain}/searches.html";
		$html = $this->browser->get_page($url);
		$html = str_replace(array("\r\n","\r","\n"), '', $html);
		
		$table = $this->parser->get_table($html, '<table cellpadding=3 cellspacing=0 border=0 bgcolor=#e8e8e8 width=520>');
		if( !array_key_exists('rows', $table) ) return $stat;
		if( !is_array($table['rows']) ) return $stat;
		// Яндекс
		// Google
		// Mail.ru
		// Bing
		foreach($table['rows'] as $row){
			if( empty($row['cells']['1']) ) continue;
			$ps = $this->parser->parse_string($row['cells']['1'], '|<td align="left"><label for="id_[^>]*">(.*?)</label></td>|');
			if($ps == '' || $ps == 'сумма выбранных' || $ps == 'всего') continue;
			$ps = str_replace( array("Search.Mail.ru"), array("Mail.ru"), $ps );
			
			$ps_today = $this->parser->parse_string($row['cells']['2'], '|<td>([\d,]+)</td>|');
			$ps_today = str_replace(',', '', $ps_today);
			
			$ps_yesterday = $this->parser->parse_string($row['cells']['4'], '|<td>([\d,]+)</td>|');
			$ps_yesterday = str_replace(',', '', $ps_yesterday);
			
			$stat[] = array(
				'ps' => $ps,
				'today' => $ps_today,
				'yesterday' => $ps_yesterday,
			);
		}
		
		if($this->use_proxy) $this->browser->shuffle_proxy();
		
		return $stat;
	}
	
	public function get_month_stat() {
		$url = "https://www.liveinternet.ru/stat/{$this->domain}/index.csv?graph=csv&period=month;total=yes";
		$csv = $this->browser->get_page($url);
		
		$arr_month = array('Янв' => 'Jan','Фев' => 'Feb','Мар' => 'Mar','Апр' => 'Apr','Май' => 'May','Июн' => 'Jun','Июл' => 'Jul','Авг' => 'Aug','Сен' => 'Sep','Окт' => 'Oct','Ноя' => 'Nov','Дек' => 'Dec');
		
		$table = explode("\n", $csv);
		foreach ($table as $k => $row) {
			if ( !$k ) continue;
			if ( !$row ) continue;
			
			list($date, $hits, $visitors) = explode(';', $row);
			
			foreach ($arr_month as $key => $value) $date = str_replace($key, $value, trim($date, '"'));
			
			$stat[] = array(
				'date' => $date,
				'hits' => $hits,
				'visitors' => $visitors
			);
		}
		
		if($this->use_proxy) $this->browser->shuffle_proxy();
		
		return json_encode($stat);
	}
	
	public function get_history() {
		$url = "https://www.liveinternet.ru/stat/{$this->domain}/index.csv?graph=csv";
		$csv = $this->browser->get_page($url);
		
		$table = explode("\n", $csv);
		foreach ($table as $k => $row) {
			if ( !$k ) continue;
			if ( !$row ) continue;
			
			list($date, $hits, $visitors) = explode(';', $row);
			$date = trim($date, '"');
		}
	}
	
}