<?php
/**
 * Парсер
 * 
 * @version  3.1.4
 */
class Parser {
	
	//**********************************//
	// ПАРСИНГ ПО РЕГУЛЯРНЫМ ВЫРАЖЕНИЯМ //
	//**********************************//
	/**
	 * Распарсить текст по указанному шаблону
	 * 
	 * Если указано multirow - функция вернёт все вхождения шаблона в тектс. Иначе - только первое вхождение.
	 * 
	 * @access  public
	 * 
	 * @param  string          $text
	 * @param  string          $pattern
	 * @param  bool[optional]  $multirow
	 * 
	 * @return  array
	 */
	public function parse_template($text, $pattern, $multirow=true) {
		
		$matches = array();
		preg_match_all('|{([\w\d\:]{1,})}|is', $pattern, $matches);
		$matches = $matches[1];
		
		$types = array(
			'digits' => '(\d*)',
			'attr'   => '([^"\'\s]+)',
			'url'    => '([^"\'\s]+)',
			'words'  => '([^<>]+)', 
			'gaps'   => '([\s\n]*)'
		);
		
		$res = array();
		foreach($matches as $key=>$match) {
			$match = explode(':', $match);
			if ( $match[0]=='space' ) continue;
			
			$res[$key+1] = $match[0];
		}
		
		$pattern = $this->shield($pattern);
		$pattern = str_replace( array(' ',"\t"), '\s?', $pattern );
		$pattern = str_replace( array("\r\n","\n","\r"), '\n?', $pattern );
		
		$text = str_replace( "\r\n", "\n", $text );
		
		foreach($matches as $key=>$match) {
			$match_t = explode(':', $match);
			if ( isset($match_t[1]) && $match_t[1] && array_key_exists($match_t[1], $types) ) {
				$replacement = $types[ $match_t[1] ];
			} else{
				$replacement = '(.*?)';
			}
			
			$pattern = str_replace('{'.$match.'}', $replacement, $pattern);
		}
		
		$pattern = '/'.$pattern.'/is';
		
		if ( $multirow ) {
			$result = $this->parse_text($text, $pattern, $res);
		} else {
			$result = $this->parse_string($text, $pattern, $res, true);
		}
		
		return $result;
	}
	
	/**
	 * Выделить из текста все совпадения с указанным шаблоном
	 * 
	 * В случае неудачи возвращает пустой массив.
	 * 
	 * @access  public
	 * 
	 * @param  string  $text
	 * @param  string  $pattern
	 * @param  array   $res
	 * 
	 * @return  array
	 */
	public function parse_text($text, $pattern, $res=array()) {
		
		$matches = array();
		preg_match_all($pattern, $text, $matches);
		
		$result = array();
		
		$count = count($matches[0]);
		for($i=0;$i<$count;$i++) {
			if ( !$res ) {
				$result[] = $matches[1][$i];
				continue;
			} elseif ( is_numeric($res) ) {
				$result[] = $matches[$res][$i];
				continue;
			}
			
			$tmp_arr = array();
			foreach($res as $key=>$value) {
				$tmp_arr[$value] = $matches[$key][$i];
			}
			$result[] = $tmp_arr;
		}
		
		return $result;
	}
	
	/**
	 * Выделить из текста первое совпадение с указанным шаблоном
	 * 
	 * @access  public
	 * 
	 * @param  string  $text
	 * @param  string  $pattern
	 * @param  array   $res
	 * @param  bool    $always_array
	 * 
	 * @return  mixed  строка или массив
	 */
	public function parse_string($text, $pattern, $res='text', $always_array=false) {
		
		$matches = array();
		preg_match($pattern, $text, $matches);
		
		if ( !$matches ) return (count($res)==1 ? '' : array());
		
		$result = array();
		
		if ( is_string($res) ) {
			if ( $always_array )
				$res = array( 1 => $res );
			else
				return $matches[1];
		} elseif ( is_numeric($res) ) {
			if ( $always_array )
				$res = array( $res => 'target' );
			else
				return $matches[$res];
		}
		
		foreach($res as $key=>$value) {
			if ( count($res)==1 && !$always_array ) return $matches[$key];
			$result[$value] = $matches[$key];
		}
		
		return $result;
	}
	
	/**
	 * Экранировать спецсимволы в шаблоне регулярного выражения
	 * 
	 * @access  public
	 * @param  string  $pattern
	 * @return  string
	 */
	public function shield($pattern) {
		$pattern = str_replace( 
			array(  '(',  ')',  '/',  '.',  '?',  '|',  '[',  ']',  '+',  '*'), 
			array( '\(', '\)', '\/', '\.', '\?', '\|', '\[', '\]', '\+', '\*'), 
			$pattern 
		);
		return $pattern;
	}
	
	
	//***********************//
	// ПОЛУЧАЕМ ЧАСТЬ ТЕКСТА //
	//***********************//
	/**
	 * Возвращает текст между первым вхождением start и первым вхождением после него end
	 * 
	 * start и end в текст не входят
	 * Если start отсутствует в тексте - будет возвращена пустая строка
	 * Если end отсутствует в тексте - при $strict==true будет возвращена пустая строка, иначе весь текст после start
	 * 
	 * @access  public
	 * 
	 * @param  string  $text
	 * @param  string  $start
	 * @param  string  $end
	 * @param  bool    $strict
	 * 
	 * @return  string
	 */
	public function get_text_between($text, $start, $end, $strict=false) {
		if ( strpos($text, $start)===false ) return '';
		$text = substr($text, strpos($text, $start) + strlen($start));
		if ( strpos($text, $end)===false ) return ($strict) ? '' : $text;
		$text = substr($text, 0, strpos($text, $end));
		return $text;
	}
	
	/**
	 * Получить содержимое указанного div с учётом вложенных тэгов
	 * 
	 * Если элементов с указанным открывающим тэгом несколько - берётся первый.
	 * Открывающий и закрывающий тэги входят в результат.
	 * 
	 * @access  public
	 * 
	 * @param  string  $text
	 * @param  string  $div   открывающий тэг (полностью - со всеми атрибутами)
	 * 
	 * @return  string
	 */
	public function get_div_content($text, $div) {
		
		$matches = array(); // подпорка
		
		$res = '';
		
		$count_ends = 0; // количество найденных закрывающих тэгов
		$count_opens = 0; // количество найденных открывающих тэгов
		
		// обрезаем начало текста
		$start = strpos($text, $div);
		if ( $start===false ) return '';
		$start += strlen($div);
		$text = substr($text, $start); 
		
		$text_last = $text; // оставшаяся часть текста
		
		while($count_opens>=$count_ends) {
			$end = strpos($text_last, '</div>'); // очередное вхождение закрывающего тэга
			$count_ends++;
			$pr_res = substr($text_last, 0, $end);
			$text_last = substr($text_last, $end+6);
			$res .= $pr_res.'</div>';
			$count_opens += preg_match_all('/<div(.*?)>/is', $pr_res, $matches);
		}
		
		$res = $div.$res;
		
		return $res;
	}
	
	/**
	 * Получить содержимое указанной таблицы с учётом вложенных тэгов
	 * 
	 * Если элементов с указанным открывающим тэгом несколько - берётся первый.
	 * Открывающий и закрывающий тэги входят в результат.
	 * 
	 * @access  public
	 * 
	 * @param  string  $text
	 * @param  string  $table  открывающий тэг (полностью - со всеми атрибутами)
	 * 
	 * @return  string
	 */
	public function get_table_content($text, $table) {
		
		$matches = array(); // подпорка
		
		$res = '';
		
		$count_ends = 0; // количество найденных закрывающих тэгов
		$count_opens = 0; // количество найденных открывающих тэгов
		
		// обрезаем начало текста
		$start = strpos($text, $table);
		if ( $start===false ) return '';
		$start += strlen($table);
		$text = substr($text, $start); 
		
		$text_last = $text; // оставшаяся часть текста
		
		while($count_opens>=$count_ends) {
			$end = strpos($text_last, '</table>'); // очередное вхождение закрывающего тэга
			$count_ends++;
			$pr_res = substr($text_last, 0, $end);
			$text_last = substr($text_last, $end+8);
			$res .= $pr_res.'</table>';
			$count_opens += preg_match_all('/<table(.*?)>/is', $pr_res, $matches);
		}
		
		$res = $table.$res;
		
		return $res;
	}
	
	/**
	 * Получить содержимое указанного тэга с учётом вложенных тэгов
	 * 
	 * Если элементов с указанным открывающим тэгом несколько - берётся первый.
	 * Открывающий и закрывающий тэги входят в результат.
	 * 
	 * @access  public
	 * 
	 * @param  string  $text
	 * @param  string  $table  открывающий тэг (полностью - со всеми атрибутами)
	 * 
	 * @return  string
	 */
	public function get_tag_content($text, $tag, $tag_end='') {
		
		$matches = array(); // подпорка
		
		$res = '';
		
		$count_ends = 0;  // количество найденных закрывающих тэгов
		$count_opens = 0; // количество найденных открывающих тэгов
		
		$general_tag = $this->parse_string($tag, '/<(\w+)[\s>]/i'); // собственно сам тэг, без атрибутов (вида div или table)
		if ( !$tag_end ) $tag_end = "</$general_tag>";           // если закрывающий тэг не указан - формируем его на основанииоткрывающего
		
		// обрезаем начало текста
		$start = strpos($text, $tag);
		if ( $start===false ) return '';
		$start += strlen($tag);
		$text = substr($text, $start); 
		
		$text_last = $text; // оставшаяся часть текста
		
		while($count_opens>=$count_ends) {
			$end = stripos($text_last, $tag_end); // очередное вхождение закрывающего тэга
			if ( $end===false ) return $tag.$text;
			++$count_ends;
			$pr_res = substr($text_last, 0, $end);
			$text_last = substr($text_last, $end+strlen($tag_end));
			$res .= $pr_res.$tag_end;
			$count_opens += preg_match_all('/<'.$general_tag.'(.*?)>/is', $pr_res, $matches);
		}
		
		$res = $tag.$res;
		
		return $res;
	}
	
	/**
	 * Получить сордержимое тэгов, у которых есть указанные атрибуты
	 * 
	 * @access  public
	 * 
	 * @param   string  $html
	 * @param   string  $tagname     имя тэга в нижнем регистре (a, table и тд)
	 * @param   array   $attrs       массив атрибутов, которые должны быть у тэга
	 * @param   bool    $only_first  вернуть только первый элемент из найденных
	 * 
	 * @return  array   массив строк, содержащих html-код всех подходящих тэгов
	 */
	public function get_tags_by_attrs($html, $tagname, $attrs, $only_first=false) {
		
		$result = array();
		
		// получаем dom-модель исходного текста
		$html = mb_convert_encoding($html, 'html-entities', 'utf-8');
		$dom = new DOMDocument();
		@$dom->loadHTML($html);
		
		// получаем все тэги указанного типа
		$tags = $dom->getElementsByTagName($tagname);
		foreach ($tags as $tag) {
			// проверяем, подходит ли тэг под указанные критерии
			// по умолчанию считаем, что тэг подходит, 
			// но как только обнаруживаем, что какой-то из нужных атрибутов в нём отсутствует или имеет иное значение, 
			// то понимаем, что тэг не подходит, и пропускаем его
			$appropriate = true;
			foreach ($attrs as $attr=>$value ) {
				$attr_value = $tag->getAttribute($attr);
				if ( !$attr_value || $attr_value!=$value ) {
					$appropriate = false;
					break;
				}
			}
			
			// если тэг подходит - возвращаем html-код тэга
			if ( $appropriate ) $result[] = $dom->saveXML($tag);
		}
		
		if ( $only_first ) return ( $result ) ? $result[0] : '';
		return $result;
	}
	
	/**
	 * Получить содержимое тэга с указанным идентификатором
	 * 
	 * @access  public
	 * 
	 * @param   string  $html
	 * @param   string  $id
	 * 
	 * @return  string
	 */
	public function get_tag_by_id($html, $id) {
		
		// получаем dom-модель исходного текста
		$html = mb_convert_encoding($html, 'html-entities', 'utf-8');
		$dom = new DOMDocument();
		@$dom->loadHTML($html);
		
		$tag = $dom->getElementById($id);
		if ( $tag ) return $dom->saveXML($tag);
		
		return '';
	}
	
	/**
	 *  Получить сордержимое тэгов с указанным классом
	 * 
	 * @access  public
	 * 
	 * @param   string  $html
	 * @param   string  $tagname     имя тэга в нижнем регистре (a, table и тд)
	 * @param   string  $class       класс, который должен быть у тэга
	 * @param   bool    $only_first  вернуть только первый элемент из найденных
	 * 
	 * @return  array   массив строк, содержащих html-код всех подходящих тэгов
	 */
	public function get_tags_by_class($html, $tagname, $class, $only_first=false) {
		$result = $this->get_tags_by_attrs($html, $tagname, array('class'=>$class), $only_first);
		return $result;
	}
	
	/**
	 * Выделить из текста указанную таблицу и разбить по строкам и ячейкам
	 * 
	 * @access  public
	 * @param   string  $html      html-код
	 * @param   string  $open_tag  открывающий тэг нужной таблицы
	 * @return  array
	 */
	public function get_table($html, $open_tag) {
		$html = $this->get_table_content($html, $open_tag);
		if ( !$html ) return array();
		
		$html = $this->clear_text($html, 'n t entities trim');
		$result = array('html' => $html, 'rows' => array());
		
		$html = mb_convert_encoding($html, 'html-entities', 'utf-8');
		
		$dom = new DOMDocument();
		@$dom->loadHTML($html);
		
		$table = $dom->getElementsByTagName('table')->item(0);
		$trs = $table->childNodes;
		foreach ($trs as $tr) {
			$res = array('html' => $dom->saveXML($tr), 'cells' => array());
			$tds = $tr->childNodes;
			foreach ($tds as $td) $res['cells'][] = $dom->saveXML($td);
			$result['rows'][] = $res;
		}
		
		return $result;
	}
	
	/**
	 * Разбить указанную таблицу по строкам и ячейкам
	 * 
	 * возвращает массив строк, каждая строка представлена массивом ячеек
	 * 
	 * @access  public
	 * @param   string  $html  html-код таблицы
	 * @return  array
	 */
	public function parse_table($html) {
		$html = $this->clear_text($html, 'n t entities trim');
		$result = array();
		
		$html = str_replace(array('<thead>','</thead>','<tbody>','</tbody>'), '', $html);
		$html = preg_replace('|<col([^>]{0,})>|', '', $html);
		$html = mb_convert_encoding($html, 'html-entities', 'utf-8');
		
		$html = str_replace(array('&times;', '&nbsp;'), array('x', ' '), $html);
		
		$dom = new DOMDocument();
		@$dom->loadHTML($html);
		
		$table = $dom->getElementsByTagName('table')->item(0);
		$trs = $table->childNodes;
		if ( !$trs ) $trs = array();
		foreach ($trs as $tr) {
			$res = array();
			$tds = $tr->childNodes;
			foreach ($tds as $td) {
				$td = $dom->saveXML($td);
				$td = $this->parse_string($td, '|<td(.*?)>(.*?)</td>|is', 2);
				$res[] = $td;
			}
			$result[] = $res;
		}
	
		return $result;
	}
	
	/**
	 * Выделить из текста список
	 * 
	 * @param  string  $html
	 * @param  string  $open_tag
	 */
	public function get_list($html, $list_type='ul') {
		if ( !$html ) return array();
		
		$html = $this->clear_text($html, 'n t entities trim');
		$result = array();
		
		$html = mb_convert_encoding($html, 'html-entities', 'utf-8');
		
		$dom = new DOMDocument();
		@$dom->loadHTML($html);
		
		$list = $dom->getElementsByTagName($list_type)->item(0);
		$items = $list->childNodes;
		foreach ($items as $item) {
			$result[] = $dom->saveXML($item);
		}
		
		return $result;
	}
	
	
	//**************//
	// ЧИСТИМ ТЕКСТ //
	//**************//
	/**
	 * Удалить все ссылки из текста
	 * 
	 * Если указан префикс - будут удалены только те ссылки, которые начинаются с него.
	 * Префикс не должен содержать экранирование специальных символов.
	 * 
	 * @access public
	 * 
	 * @param  string  $html
	 * @param  string[optional]  $prefix
	 * 
	 * @return  string
	 */
	public function unactive_links($html, $prefix='') {
		if ( !$prefix ) { // префикс не указан - удаляем все ссылки
			$html = preg_replace('/<a([^>]{1,})>/is', '', $html);
			$html = str_ireplace('</a>', '', $html);
		} else { // префикс указан - удаляем только те ссылки, которые начинаются с него
			// FIXME: удаляет только те ссылки, которые заключены в ""
			$prefix = $this->shield($prefix);
			$html = preg_replace('/<a href="'.$prefix.'(.*?)"(.*?)>(.*?)<\/a>/is', '$3', $html);
		}
		return $html;
	}
	
	/**
	 * Заменить доменное имя во всех абсолютных ссылках на указанное
	 * 
	 * @param  string  $html
	 * @param  string  $domain
	 * 
	 * @return  string
	 */
	public function change_links_domain($html, $domain) {
		$links = $this->get_links($html);
		
		foreach ($links as $text=>$link) {
			$anchor = parse_url($link, PHP_URL_FRAGMENT);
			$new_link = ($anchor) ? $domain.'#'.$anchor : $domain;
			
			$html = str_replace($link, $new_link, $html);
			if ( stripos($text, 'http://')!==false ) str_replace($text, $new_link, $html);
		}
		
		return $html;
	}
	
	/**
	 * Удалить из текста все ссылки
	 */
	public function delete_links($html) {
		
		$html = $this->unactive_links($html);
		$content = strip_tags($html);
		
		$links = $this->parse_text($html, "#((http|https|ftp)://(\S*?\.\S*?))(\s|\;|\)|\]|\[|\{|\}|,|\"|'|:|\<|$|\.\s)#i", 1);
		foreach ($links as $link) if ( strpos($content, $link)!==false ) $html = str_replace($link, ' ', $html);
		
		$links = $this->parse_text($html, "#(www\.(\S*?\.\S*?))(\s|\;|\)|\]|\[|\{|\}|,|\"|'|:|\<|$|\.\s)#i", 1);
		foreach ($links as $link) if ( strpos($content, $link)!==false ) $html = str_replace($link, ' ', $html);
		
		return $html;
	}
	
	/**
	 * Удалить все скрипты из текста
	 * 
	 * Удаляет всё содерджимое тэгов <script>
	 * 
	 * @access  public
	 * @param   string  $html
	 * @return  string
	 */
	public function delete_scripts($html) {
		$html = preg_replace('|<script(.*?)</script>|is', '', $html);
		return $html;
	}
	
	/**
	 * Удалить из текста html-комментарии
	 * 
	 * @access  public
	 * @param   string  $html
	 * @return  string
	 */
	public function delete_comms($html) {
		$html = preg_replace('/<!--.*-->/Uis', '', $html);
		return $html;
	}
	
	/**
	 * Удалить из текста все символы перевода строк
	 * 
	 * @access  public
	 * @param  string  $html
	 * @return  string
	 */
	public function delete_n_chars($html) {
		$html = str_replace( array("\r\n","\r","\n"), '', $html );
		return $html;
	}
	
	/**
	 * Удалить из текста смежные пробелы и табуляцию
	 * 
	 * @access  public
	 * @param   string  $html
	 * @return  string
	 */
	public function delete_t_chars($html) {
		$html = str_replace("\t",' ',$html);
		while ( strpos($html,'  ')!==false ) $html = str_replace('  ',' ',$html);
		$html = str_replace('> <', '><', $html);
		return $html;
	}
	
	/**
	 * Удалить атрибуты из тэгов в тексте
 	 * 
 	 * @access  public
	 * @param  string  $html
	 * @return  string
	 */
	public function delete_attributes($html) {
		$spec_tags = array(    // список тэгов, для которых надо оставить атрибуты (если список атрибутов пуст - оставляем все атрибуты)
			'img'    => array('src'),
			'a'      => array('href'),
			'td'     => array('colspan', 'rowspan'),
			'input'  => array(),
			'option' => array('value'),
			'object' => array(),
			'param'  => array(),
			'embed'  => array()
		);
		
		$html = $this->normalize_tags($html, $spec_tags, true);
		return $html;
	}
	
	/**
	 * Удалить из текста перечисленные сущности (обработка - слева направо)
	 * 
	 * @access public
	 * 
	 * @param  string  $html
	 * @param  string  $flags
	 * @param  string  $allowable_tags
	 * 
	 * @return  string
	 */
	public function clear_text($html, $flags='entities tags n t trim', $allowable_tags=null) {
		$flags = explode(' ', $flags);
		for($i=0;$i<count($flags);$i++) {
			$flag = $flags[$i];
			if ($flag=='entities') $html = html_entity_decode($html, ENT_QUOTES, 'UTF-8');
			if ($flag=='tags')     $html = strip_tags($html, $allowable_tags);
			if ($flag=='n')        $html = $this->delete_n_chars($html);
			if ($flag=='t')        $html = $this->delete_t_chars($html);
			if ($flag=='trim')     $html = trim($html);
			if ($flag=='attrs')    $html = $this->delete_attributes($html);
		}
		return $html;
	}
	
	
	//*****************************************//
	// ГРУЗИМ ВСЕ ИЗОБРАЖЕНИЯ ИЗ ТЕКСТА К СЕБЕ // 
	//*****************************************//
	/** Получить адреса всех картинок из текста
	 * Если указан параметр url - все адреса будут нормализованы относительно его
	 * 
	 * @access  public
	 * 
	 * @param  string            $html
	 * @param  string[optional]  $url
	 * 
	 * @return  array
	 */
	public function get_images($html, $url='') {
		$images = array();
		
		$imgs = $this->parse_text($html, '/<img([^>]+)>/is');
		foreach($imgs as $img) {
			$src = $this->parse_string($img, '/src="(.*?)"/is');
			if ( !$src ) $src = $this->parse_string($img, "/src='(.*?)'/is");
			if ( !$src ) $src = $this->parse_string($img, "/src=([^\s]+)/is");
			
			if ($url) $src = $this->normalize_link($src, $url);
			
			$images[] = $src;
		}
		
		return $images;
	}
	
	/**
	 * Скачать из html-кода все картинки в указанную папку
	 * 
	 * Картинкам прописываются новые пути.
	 * 
	 * @access  public
	 * 
	 * @param  string  $text
	 * @param  string  $pic_dir
	 * @param  string  $watermark
	 * @param  string  $url
	 * @param  string  $not_repeat
	 * 
	 * @return  string
	 */
	public function load_images($text, $pic_dir, $watermark=false, $url='', $not_repeat=true, $resize=0, $file_path='') {
		$imgs = $this->get_images($text);
		
		if ( $url ) {
			$url = $this->normalize_url($url);
			$site = $this->parse_string($url, '/^http:\/\/([-\w\d\.]+)/');
			$site = 'http://'.$site;
			$cat = dirname($url);
		}
		
		foreach($imgs as $src) {
			
			$old_link = $src;
			$new_link = '';
			
			if ( $url ) {
				if ( $this->string_start($src, 'http://') ) $new_link = $src;
				elseif ( $src{0}=='/' )  $new_link = $site . $src;
				else $new_link = $cat.'/'.$src;
			} else {
				$new_link = $src;
			}
			if ( !$new_link ) continue;
			if ( !$this->string_start($new_link, 'http://') ) continue;
			
			$img = new Image($new_link);
			$res = $img->load($pic_dir, $not_repeat, md5($new_link).'.jpg');
			if ( $res && $resize ) $img->resize($resize, $resize);
			if ( $res && $watermark ) $img->watermark($watermark);
			if ( !$res ) $text = preg_replace('/<img([^>]{0,})'.$this->shield($old_link).'([^>]{0,})>/is', '', $text);
			
			$new_url = str_replace($pic_dir, $file_path, $img->url);
			$text = str_replace($old_link, $new_url, $text);
		}
		
		return $text;
	}
	
	
	//*****************//
	// ОБРАБОТКА СТРОК //
	//*****************//
	/**
	 * Проверить, начинается ли строка с указанного префикса
	 * FIXME: с кириллицей скорее всего работать не будет
	 * 
	 * @access  public
	 * 
	 * @param  string  $string
	 * @param  string  $start
	 * @param  bool    $register
	 * 
	 * @return  bool
	 */
	public function string_start($string, $start, $register=true) {
		if ( $register ) {
			if ( strtolower(substr($string, 0, strlen($start)))==strtolower($start) ) return true;
		} else {
			if ( substr($string, 0, strlen($start))==$start ) return true;
		}
		return false;
	}
	
	/**
	 * Проверить, оканчивается ли строка указанной подстрокой
	 * 
	 * @access  public
	 * 
	 * @param   string  $string
	 * @param   string  $end
	 * 
	 * @return  bool
	 */
	public function string_end($string, $end) {
		$substr = substr($string, strlen($string)-strlen($end));
		if ( $substr==$end ) return true;
		return false; 
	}
	
	/**
	 * Проверить, содержится ли подстрока в строке
	 * 
	 * возвращает подстроку или false
	 * $substr - либо строка, либо массив; для массива возвращается первый входящий вариант
	 * 
	 * @access  public
	 * 
	 * @param   string        $string
	 * @param   string|array  $substr
	 * 
	 * @return  string
	 */
	public function in_string($string, $substr) {
		
		if ( is_array($substr) ) { # поисковый запрос - массив
			
			foreach($substr as $sub) {
				$res = strpos($string, $sub);
				if ($res!==false) return $sub;
			}
			
		} else { # поисковый запрос - строка
			
			$res = strpos($string, $substr);
			if ($res!==false) return $substr;
			
		}
		
		return false;
	}
	
	/**
	 * Получить все слова из строки
	 * 
	 * @access  public
	 * @param  string  $string
	 * @return  array
	 */
	public function get_words($string) {
		
		$quotes = array ("\x22", "\x60", "\t", '\n', '\r', "\n", "\r", '\\', ",", ".", "/", "¬", "#", ";", ":", "@", "~", "[", "]", "{", "}", "=", "-", "+", ")", "(", "*", "^", "%", "$", "<", ">", "?", "!", '"' );
		
		$string = str_replace("&nbsp;", " ", $string);
		$string = str_replace('<br />', ' ', $string);
		$string = trim( strip_tags( $string ) );
		$string = str_replace( $quotes, '', $string);
		
		$words = explode(' ', $string);
		
		return $words;
	}
	
	/**
	 * Нормализовать url-адрес
	 * 
	 * Добавляем http:// если нужно
	 * 
	 * @param   string  $url
	 * @return  string
	 */
	public function normalize_url($url) {
		$url = trim($url);
		if ( !$this->string_start($url, 'http://') && !$this->string_start($url, 'https://') ) $url = 'http://'.$url;
		$url = str_replace('&amp;', '&', $url);
		return $url;
	}
	
	/**
	 * Сделать ссылку абсолютной
	 * 
	 * @param  string  $link  ссылка, которую надо преобразовать
	 * @param  string  $url   адрес страницы, с которой была взята ссылка
	 * 
	 * @return  string
	 */
	public function normalize_link($link, $url) {
		
		$link = str_replace('&amp;', '&', $link);
		
		// не обрабатываем
		if ( $this->string_start($link, 'http://') )     return $link; // абсолютные ссылки
		if ( $this->string_start($link, 'https://') )     return $link; // абсолютные ссылки
		if ( $this->string_start($link, 'javascript:') ) return $link; // javascript
		if ( $this->string_start($link, 'mailto:') )     return $link; // адрес почты
		
		$url    = $this->normalize_url($url);
		$domain = parse_url($url, PHP_URL_HOST);
		$scheme = parse_url($url, PHP_URL_SCHEME);
		
		// если ссылка - якорь, то просто добавляем его к адресу страницы
		if ( $link{0}=='#' ) return $url.$link;
		
		// если ссылка начинается на '/' - она прописана относительно домена сайта
		if ( $link{0}=='/' ) {
			$link = $scheme.'://'.$domain.$link;
			return $link;
		}
		
		// относительная ссылка
		// TODO: это как-то проще можно сделать
		$cat = $this->parse_string($url, '/^http[s]{0,}:\/\/(.*?)$/');
		$cat = dirname($cat);
		if ( $cat=='.' || $cat=='' ) $cat = $domain;
		$cat = explode('/', $cat);
		$level_up = substr_count($link, '../');
		for($i=count($cat)-1, $j=0; $j<$level_up && $i>=1;$i--,$j++) unset($cat[$i]);
		$cat = implode('/', $cat);
		$link = str_replace('../', '', $link);
		$link = 'http://'.$cat.'/'.$link;
		$link = str_replace('./', '', $link);
		
		return $link;
	}
	
	/**
	 * Превращаем все ссылки в тексте в абсолютные
	 * FIXME: получает ссылки только в двойных кавычках
	 * 
	 * @access  public
	 * 
	 * @param   string  $html  текст страницы
	 * @param   string  $url   адрес страницы
	 * 
	 * @return  string         страница с абсолютными ссылками
	 */
	public function absolute_urls($html, $url) {
		
		$site = $this->parse_string($url, '/^http:\/\/([-\w\d\.]+)/');
		$site = 'http://'.$site;
		
		$cat = dirname($url);
		if ( $cat=='http:' ) $cat = $url;
		
		$src = $this->parse_text($html, '/src="([^"]+)"/is');
		$href = $this->parse_text($html, '/href="([^"]+)"/is');
		
		$links = array_merge($src, $href);
		$links = array_unique($links);
		
		$old_links = array();
		$new_links = array();
		
		foreach($links as $link) {
			if ( $link=='#' ) continue;
			$old_links[] = $link;
			$new_links[] = $this->normalize_link($link, $url);
		}
		
		$html = str_replace($old_links, $new_links, $html);
		
		return $html;
	}
	
	/**
	 * Получить количество слов в тексте
	 * 
	 * @param  string  $text
	 * @return  int
	 */
	public function cnt_words($text) {
		$text = $this->clear_text($text);
		$words = explode(' ', $text);
		return count($words);
	}
	
	
	//*********************//
	// ОБРАБОТКА HTML-КОДА //
	//*********************//
	/**
	 * Получить все ссылки из текста
	 * 
	 * Если указан параметр $url - все ссылки будут нормализованы относительно его
	 * 
	 * @param  string  $html
	 * @param  string[optional]  $url
	 */
	public function get_links($html, $url='') {
		
		$result = array();
		
		// получаем dom-модель исходного текста
		$html = mb_convert_encoding($html, 'html-entities', 'utf-8');
		$dom = new DOMDocument();
		@$dom->loadHTML($html);
		
		// получаем все ссылки из текста
		$tags = $dom->getElementsByTagName('a');
		foreach ($tags as $tag) {
			$href = $tag->getAttribute('href');
			
			if ( !$href ) continue;
			if ( $this->string_start($href, 'javascript:') ) continue;
			if ( $this->string_start($href, 'mailto:') ) continue;
			
			if ( $url ) $href = $this->normalize_link($href, $url);
			if ( !in_array($href, $result) ) {
				if ( $tag->nodeValue )
					$result[ $tag->nodeValue ] = $href;
				else
					$result[] = $href;
			}
		}
		
		return $result;
	}
	
	/**
	 * Удалить из текста содержимое указанного тэга
	 * 
	 * @access  public
	 * 
	 * @param  string  $html  
	 * @param  string  $tag   открывающий тэг (полностью - <h3 id="title">)
	 * 
	 * @return  string
	 */
	public function delete_tag($html, $tag) {
		$string = $this->get_tag_content($html, $tag);
		$html = str_replace($string, '', $html);
		return $html;
	}
	
	/**
	 * Получить все скрипты из текста
	 * 
	 * Возвращает всё содержимое тэгов <script>
	 * Результат обрамлён в тэги <script>
	 * 
	 * @access  public
	 * @param   string  $html
	 * @return  array
	 */
	public function get_scripts($html) {
		
		$scripts = $this->parse_text($html, '|<script(.*?)</script>|is');
		$scripts = array_unique($scripts);
		
		for ($i = 0; $i < count($scripts); $i++) {
			$scripts[$i] = '<script'.$scripts[$i].'</script>';
		}
		
		return $scripts;
	}
	
	/**
	 * Получить ссылки на js-файлы
	 * 
	 * @access  public
	 * @param  string  $html
	 * @return  array
	 */
	public function get_js_files($html) {
		$links = array();
		
		$src = $this->parse_text($html, '/src="(.*?)"/i');
		foreach ($src as $link) {
			if ( $this->get_extension($link)!='js' ) continue;
			$links[] = $link;
		}
		$links = array_unique($links);
		
		return $links;
	}
	
	/**
	 * Получить ссылки на css-файлы
	 * 
	 * @access  public
	 * @param  string  $html
	 * @return  array
	 */
	public function get_css_files($html) {
		$links = array();
		
		$src = $this->parse_text($html, '/href="(.*?)"/i');
		foreach ($src as $link) {
			if ( $this->get_extension($link)!='css' ) continue;
			$links[] = $link;
		}
		$links = array_unique($links);
		
		return $links;
	}
	
	/**
	 * Заменить в тексте указанные строки на их md5-суммы, обрамлённые в @
	 * 
	 * @access  public
	 * 
	 * @param  string  $html    текст, в котором надо "спрятать" строки 
	 * @param  array   $blocks  массив строк, которые надо спрятать
	 * 
	 * @return  string
	 */
	public function mask_blocks($html, $blocks) {
		foreach ($blocks as $block) {
			$md5 = '@'.md5($block).'@';
			$html = str_replace($block, $md5, $html);
		}
		return $html;
	}
	
	/**
	 * Восстановить в тексте указанные строки
	 * 
	 * @access public
	 * 
	 * @param  string  $html    текст со "спрятанными" строками
	 * @param  array   $blocks  массив строк, которые были "спрятаны"
	 * 
	 * @return  string
	 */
	public function unmask_blocks($html, $blocks) {
		foreach ($blocks as $block) {
			$md5 = '@'.md5($block).'@';
			$html = str_replace($md5, $block, $html);
		}
		return $html;
	}
	
	/**
	 * Получить расширение файла
	 * Расширение определяется как часть строки после последней точки.
	 * 
	 * @access  private
	 * @param   string  $filename
	 * @return  string
	 */
	private function get_extension($filename) {
		$filename = basename($filename);
		return end(explode('.', $filename));
	}
	
	
	//*************************//
	// ПРЕОБРАЗОВАНИЕ ЗНАЧЕНИЙ //
	//*************************//
	/**
	 * Преобразовать значение в целое число
	 */
	public function to_int($value, $default=0) {
	
		if ( !$value ) return $default;
		if ( is_object($value) ) return $default;
		
		if ( is_int($value) ) return $value;
		
		$value = intval($value);
		if ( !$value ) return $default;
		
		return $value;
	}
	
	/**
	 * Преобразовать значение в строку
	 */
	public function to_string($value, $clear=false, $default='') {
		
		if ( !$value ) return $default;
		if ( is_object($value) ) return $default;
		
		if ( is_string($value) ) {
			if ( $clear ) $value = $this->clear_text($value);
			return $value;
		}
		
		$value = strval($value);
		if ( !$value ) return $default;
		if ( $clear ) $value = $this->clear_text($value);
		
		return $value;
	}
	
	/**
	 * Преобразовать значение в массив
	 */
	public function to_array($value, $delimiter=',', $default=array()) {
		
		if ( is_array($value) ) return $value;
		
		if ( !$value ) return $default;
		if ( is_object($value) ) return $default;
		
		$array = array();
		$items = explode($delimiter, $value);
		foreach ($items as $item) {
			$item = trim($item);
			$item = delete_n_t($item);
			
			$array[] = $item;
		}
		
		return $array;
	}
	
	/**
	 * Очистить все элементы массива от спецсимволов
	 */
	public function clear_array( $array, $save_keys=true, $flags=false, $force=false ) {
		
		$result = array();
		
		foreach ($array as $key => $item) {
			if ( is_array($item) || is_object($item) ) {
				if ( $force ) $item = '';
			} else {
				$item = ( $flags ) ? $this->clear_text($item, $flags) : $this->clear_text($item);
			}
			
			if ( $save_keys )
				$result[ $key ] = $item;
			else
				$result[] = $item;
		}
		
		return $result;
	}
	
	
	//***********************************//
	// ПРИВОДИМ ТЭГИ К СТАНДАРТНОМУ ВИДУ //
	//***********************************//
	/**
	 * Привести все тэги в тексте к нормальному виду 
	 * 
	 * Избавляемся от лишних атрибутов, приводим тэги к нижнему регистру, значения атрибутов заключаем в ""
	 * 
	 * @access  private
	 * @param   string  $html
	 * @param   array   $spec_tags
	 * @param   bool    $use_spec_tags
	 * @return  string
	 */
	private function normalize_tags($html, $spec_tags, $use_spec_tags) {
		
		$tags = $this->get_tags($html);
		
		foreach($tags as $tag) {
			$new_tag = $tag['tag'];
			$old_tag = '<'.$tag['primary_tag'].'>';
			
			// обрабатываем атрибуты тэга, если нужно
			if ( (!$use_spec_tags) || (array_key_exists($tag['tag'], $spec_tags) && $tag['attrs']) ) {
				$attrs = array();
				foreach($tag['attrs'] as $name=>$value) {
					$name = strtolower($name);
					if ( $use_spec_tags && count($spec_tags[$tag['tag']])>0 && !in_array($name, $spec_tags[$tag['tag']]) ) continue;
					$attrs[] = $name . '="' . $value . '"';
				} 
				$attrs = implode(' ', $attrs);
				if ( $attrs ) $new_tag .= ' '.$attrs;
			}
			
			$new_tag = ($tag['self_closed']) ? "<$new_tag />" : "<$new_tag>";
			
			$html = str_replace($old_tag, $new_tag, $html);
		}
		
		return $html;
	}
	
	/**
	 * Получить все тэги из текста
	 * 
	 * Возвращает массив вида
	 * [key]=>( 
	 * 		primary_tag => исходный тэг без <>
	 * 		tag => тип тэга 
	 * 		attrs => массив атрибутов вида [name]=>value 
	 * 		self_closed => bool
	 * 	)
	 * 
	 * @access  private
	 * @param   string  $html
	 * @return  array
	 */
	private function get_tags($html) {
		
		$tags = array();
		
		// получаем все тэги из текста
		$primary_tags = $this->parse_text($html, '/<([\/\!]*?[^<>]*?)>/is');
		$primary_tags = array_unique($primary_tags);
		
		// обрабатываемый каждый тэг
		foreach($primary_tags as $primary_tag) {
			$tag = array(
				'primary_tag' => $primary_tag,
				'tag' => '',
				'attrs' => array(),
				'self_closed' => false
			);
			
			if ( $primary_tag{0}=='/' ) { # закрывающий тэг - с ним манипуляции не нужны, поскольку атрибутов такие тэги не содержат
				$tag['tag'] = strtolower($primary_tag);
			} else {
				// выделяем тип тэга и его атрибуты
				$m = explode(' ', $primary_tag, 2);
				$tag['tag'] = $m[0];
				$tag['attrs'] = ( isset($m[1]) ) ? $m[1] : '';
				
				$tag['tag'] = strtolower($tag['tag']);
				
				// проверяем, является ли тэг самозакрывающимся - то есть нужен ли слэш на конце тэга
				if ( $tag['attrs']=='' ) { # атрибутов нет - тэг обычный
					$tag['self_closed'] = false;
				} else { # атрибуты есть - надо проверить слэш на конце; заодно распарсим атрибуты
					$tag['self_closed'] = ( $tag['attrs']{strlen($tag['attrs'])-1}=='/' ) ? true : false ;
					$tag['attrs'] = $this->parse_attributes($tag['attrs']);
				}
			}
			
			$tags[] = $tag;
		}
		
		return $tags;
	}
	
	/**
	 * Превратить строку с атрибутами в ассоциативный массив вида [имя_атрибута]=>значение_атрибута
	 * 
	 * @access  private
	 * @param   string  $attr_str
	 * @return  array
	 */
	private function parse_attributes($attr_str) {
		
		if ( $attr_str{strlen($attr_str)-1}=='/' ) $attr_str = substr($attr_str, 0, strlen($attr_str)-1); // случай <tag /> - удаляем закрывающий слэш из строки
		$attr_str = trim($attr_str);
		
		$attrs = array();
		
		$state = 0; // 0 - разделитель, 1 - имя атрибута, 2 - значение атрибута
		$flag_quote = ''; // закрывающий символ для значения атрибута (", ' или пробел)
		
		$curr_attr = '';
		$curr_value = '';
		
		$attr_str = $attr_str.' '; // значение последнего атрибута гарантированно заканчивается пробелом
		
		for($i=0;$i<strlen($attr_str);$i++) {
			$sym = $attr_str{$i};
			
			switch( $state ) {
				case 0:
					if ( $sym==' ' || $sym=="\t" ) continue;
					
					$curr_attr = $sym;
					$state = 1;
					
					break;
					
				case 1:
					
					if ( $sym=='=' ) { // имя атрибута закончилось - дальше идёт его значение
						$state = 2;
						continue;
					}
					
					if ( $sym==' ' || $sym=="\t" ) { // имя атрибута закончилось, на значения у него нет
						$attrs[$curr_attr] = '';
						$curr_attr = '';
						$state = 0;
						continue;
					}
					
					$curr_attr .= $sym;
					
					break;
					
				case 2:
					
					if ( $curr_value=='' && !$flag_quote ) { // только начали парсить значение - определяемся с закрывающим символом
						if ( $sym=='"' ) $flag_quote = '"';
						elseif ( $sym=="'" ) $flag_quote = "'";
						else {
							$flag_quote = ' ';
							$curr_value = $sym;
						}
						
						continue;
					}
					
					if ( $sym==$flag_quote ) {
						$attrs[$curr_attr] = $curr_value;
						
						$flag_quote = '';
						$curr_attr = '';
						$curr_value = '';
						
						$state = 0;
						continue;
					}
					
					$curr_value .= $sym;
					
					break;
			}
		}
		
		return $attrs;
	}
	
}
