<?php
/**
 * Набор полезных функций
 * 
 * @version  3.0.4
 */

//********************//
// РАБОТА СО СТРОКАМИ //
//********************//
/**
 * Получить случайную строку из больших и малых латинских букв, цифр, указанной длины
 * 
 * @param int $count
 * @return string
 */
function random_str($length) {
	$abc = 'abcdefghijklmnopqrstuvwxyz1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZ';
	$rand_max = strlen($abc)-1;
	$str = '';
	for($i=0;$i<$length;$i++) {
		$rand = rand(0,$rand_max);
		$letter = $abc{$rand};
		$str .= $letter;
	}
	return($str);
}

/**
 * Получить время
 */
function time_elapsed_string($ftime,$ptime){
    $etime =  $ptime - $ftime;

    if ($etime < 1)
    {
        return '0 секунд';
    }

    $a = array(
                12 * 30 * 24 * 60 * 60  =>  array('год','года','лет'),
                30 * 24 * 60 * 60       =>  array('месяц','месяца','месяцев'),
                24 * 60 * 60            =>  array('день','дня','дней'),
                60 * 60                 =>  array('час','часа','часов'),
                60                      =>  array('минута','минуты','минут'),
                1                       =>  array('секунда','секунды','секунд')
                );

    foreach ($a as $secs => $str)
    {
        $d = $etime / $secs;
        if ($d >= 1)
        {
            $r = round($d);
            return declOfNum($r,$str);
        }
    }
}

function declOfNum($number, $titles)
{
    $cases = array (2, 0, 1, 1, 1, 2);
    return $number." ".$titles[ ($number%100>4 && $number%100<20)? 2 : $cases[min($number%10, 5)] ];
}

/**
 * Получить уникальный идентификатор из латинских букв и цифр
 */
function get_uniq_id() {
	return( md5(uniqid(random_str(15), true)) );
}

/**
 * Преобразовать все латинские буквы в нижний регистр, не затрагивая кириллицу и любые другие символы
 * @param string $str
 * @return string
 */
function strtolower_latin($str) {
	$upper = 'QWERTYUIOPASDFGHJKLZXCVBNM';
	$lower = 'qwertyuiopasdfghjklzxcvbnm';
	$str = strtr($str, $upper, $lower);
	return $str;
}

/**
 * Привести строку в кодировке utf-8 к нижнему регистру
 * 
 * Корректно обрабатывает кириллицу.
 * 
 * @param string $str
 * @return string
 */
function strtolower_cyr($text) {
	
	$abc = array(
			'Й' => 'й', 'Ц' => 'ц', 'У' => 'у',
			'К' => 'к', 'Е' => 'е', 'Ё' => 'ё', 'Н' => 'н', 'Г' => 'г', 'Ш' => 'ш', 'Щ' => 'щ', 'З' => 'з', 'Х' => 'х',
			'Ъ' => 'ъ', 'Ф' => 'ф', 'Ы' => 'ы', 'В' => 'в', 'А' => 'а', 'П' => 'п', 'Р' => 'р', 'О' => 'о', 'Л' => 'л',
			'Д' => 'д', 'Ж' => 'ж', 'Э' => 'э', 'Я' => 'я', 'Ч' => 'ч', 'С' => 'с', 'М' => 'м', 'И' => 'и', 'Т' => 'т',
			'Ь' => 'ь', 'Б' => 'б', 'Ю' => 'ю'
	);
	
	foreach ($abc as $upper=>$lower) $text = str_replace($upper, $lower, $text);
	return $text;
}

/**
 * Транслитерация строки
 * 
 * Удаляет тэги.
 * Преобразует html-сущности в символы.
 * Заменяет русские буквы на английские аналоги.
 * Если требуется - удаляет из текста всё, кроме английских букв, цифр и символа подчёркивания (именно удаляет, а не заменяет)
 * 
 * @param  string  $words   строка
 * @param  bool    $strong  нужно ли удалять из строки неалфавитные символы
 * @param  string  $space   на какой символ нужно заменить пробел (если не указан - пробел не обрабатывается)
 * 
 * @return  string
 */
function translit($words, $strong=true, $space=' ') {
	
	$words = html_entity_decode($words, ENT_QUOTES, 'UTF-8');
	$words = strip_tags($words);
	$words = str_replace(array("\t","\r\n","\r","\n",'%20'), ' ', $words);
	$words = trim($words);
	
	$abc = array(
			'й' => 'j', 'ц' => 'ts', 'у' => 'u', 'к' => 'k', 'е' => 'e', 'ё' => 'jo', 'н' => 'n', 'г' => 'g', 'ш' => 'sh',
			'щ' => 'sh', 'з' => 'z', 'х' => 'h', 'ъ' => '', 'ф' => 'f', 'ы' => 'i', 'в' => 'v', 'а' => 'a', 'п' => 'p',
			'р' => 'r', 'о' => 'o', 'л' => 'l', 'д' => 'd', 'ж' => 'zh', 'э' => 'e', 'я' => 'ja', 'ч' => 'ch', 'с' => 's',
			'м' => 'm', 'и' => 'i', 'т' => 't', 'ь' => '', 'б' => 'b', 'ю' => 'u', 'Й' => 'j', 'Ц' => 'ts', 'У' => 'u',
			'К' => 'k', 'Е' => 'e', 'Ё' => 'jo', 'Н' => 'n', 'Г' => 'g', 'Ш' => 'sh', 'Щ' => 'sh', 'З' => 'z', 'Х' => 'h',
			'Ъ' => '', 'Ф' => 'f', 'Ы' => 'i', 'В' => 'v', 'А' => 'a', 'П' => 'p', 'Р' => 'r', 'О' => 'o', 'Л' => 'l',
			'Д' => 'd', 'Ж' => 'zh', 'Э' => 'e', 'Я' => 'ja', 'Ч' => 'ch', 'С' => 's', 'М' => 'm', 'И' => 'i', 'Т' => 't',
			'Ь' => '', 'Б' => 'b', 'Ю' => 'u'
	);
	foreach($abc as $sym=>$translit) $words = str_replace($sym, $translit, $words);
	
	if ( $space && $space!=' ' ) $words = str_replace(' ', $space, $words);
	
	if ( $strong ) {
		$words = preg_replace('|[^-\d\sa-zA-Z]|iu', '-', $words);
		$words = preg_replace('|-{2,}|', '-', $words);
		$words = strtolower($words);
	}
	
	$words = trim($words, ' -');
	
	return $words;
}

/**
 * Обрезать строку до указанной длины по ближайшему подходящему символу
 * 
 * @param string $string   строка
 * @param int    $length   до какой длины обрезать строку
 * @param array  $symbols  список символов, по которым должна быть обрезана строка
 * @param bool   $more     строка должна быть больше (true) или меньше (false) указанной длины
 * 
 * @return string
 */
function cut_string($string, $length, $symbols=array(',','.','!','?',' '), $more=true) {
	
	if ( mstrlen($string)<=$length ) return $string;
	
	$result = msubstr($string, 0, $length);
	$other  = msubstr($string, $length);
	
	$sizes = array();
	foreach($symbols as $symbol) {
		$size = ($more) ? mstrpos($other,$symbol) : mstrrpos($result,$symbol);
		if ( false===$size ) continue;
		$sizes[] = $size;
	}
	
	if ( !$sizes ) return $result;
	
	sort($sizes);
	$size = ($more) ? $sizes[0] : end($sizes);
	
	if ( !$size ) return $result;
	
	if ( $more ) {
		$other = msubstr($other, 0, $size+1);
		$result .= $other;
	} else {
		$result = msubstr($result, 0, $size);
	}
	
	return $result;
}

/**
 * Удалить с концов строки непечатаемые символы
 * 
 * @param   string  $text
 * @return  string
 */
function ext_trim($text) {
	$text = basename(trim($text, "\x00..\x1F"));
	return $text;
}

/**
 * Проверка валидности email
 * @param string $email
 * @return bool
 */
function correct_email($email) {
	if ( eregi("^[a-zA-Z0-9\._-]+@[a-zA-Z0-9\._-]+\.[a-zA-Z]{2,4}$",$email) ) {
		return(true);
	}
	return(false);
}

/**
 * Выделить из url домен
 * @param   string  $url
 * @return  string  $domain
 */
function get_domain($url) {
	$domain = parse_url($url, PHP_URL_HOST);
	
	$domain = strtolower($domain);
	$domain = str_replace('www.', '', $domain);
	
	return $domain;
}

/**
 * Получить текст между первым вхождением start и первым вхождением после него end, start и end в текст не входят
 * 
 * @param  string  $text
 * @param  string  $start
 * @param  string  $end
 * @return  string
 */
function get_text_between($text, $start, $end) {
	$res1 = substr($text, strpos($text, $start) + strlen($start));
	$res2 = substr($res1, 0, strpos($res1, $end));
	
	return($res2);
}

/**
 * Преобразовать первую букву строки в верхний регистр
 * 
 * @param  string  $string
 * @return  string
 */
function first_letter_to_upper($string) {
	
	$abc = array(
			'й' => 'Й', 'ц' => 'Ц', 'у' => 'У', 'к' => 'К', 'е' => 'Е', 'ё' => 'Ё', 'н' => 'Н', 'г' => 'Г', 'Ш' => 'Ш',
			'щ' => 'Щ', 'з' => 'З', 'х' => 'Х', 'ф' => 'Ф', 'в' => 'В', 'а' => 'А', 'п' => 'П', 'р' => 'Р', 'о' => 'О', 
			'л' => 'Л', 'д' => 'Д', 'ж' => 'Ж', 'э' => 'Э', 'я' => 'Я', 'ч' => 'Ч', 'с' => 'С',	'м' => 'М', 'и' => 'И', 
			'т' => 'Т', 'б' => 'Б', 'ю' => 'Ю',
			'q' => 'Q', 'w' => 'W', 'e' => 'E', 'r' => 'R', 't' => 'T', 'y' => 'Y', 'u' => 'U', 'i' => 'I', 'o' => 'O', 
			'p' => 'P', 'a' => 'A', 's' => 'S', 'd' => 'D', 'f' => 'F', 'g' => 'G', 'h' => 'H', 'j' => 'J', 'k' => 'K', 
			'l' => 'L', 'z' => 'Z', 'x' => 'X', 'c' => 'C', 'v' => 'V', 'b' => 'B', 'n' => 'N', 'm' => 'M'
	);
	
	$letter = $string{0};
	if ( isset($abc[$letter]) ) {
		$string = substr_replace($string, $abc[$letter], 0, 1);
	} else {
		$letter = substr($string, 0, 2);
		if ( isset($abc[$letter]) ) $string = substr_replace($string, $abc[$letter], 0, 2);
	}
	
	return $string;
}

/**
 * Получить длину строки в произвольной кодировке (windows-1251, utf-8)
 * 
 * @param  string  $string
 * @return  int
 */
function mstrlen($string) {
	$charset = detect_encoding($string);
	if ( $charset=='windows-1251' ) return strlen($string);
	
	if ( function_exists('mb_strlen') ) return mb_strlen($string, 'utf-8');
	if ( function_exists('iconv_strlen') ) return iconv_strlen($string, 'utf-8');
	
	$string = ic_uw($string);
	return strlen($string);
}

/**
 * Получить подстроку из строки в произвольной кодировке (windows-1251, utf-8)
 * 
 * @param  string  $string
 * @param  int  $start
 * @param  int  $length
 * @return  string
 */
function msubstr($string, $start, $length=0) {
	if ( $length==0 ) $length = mstrlen($string);
	
	$charset = detect_encoding($string);
	if ( $charset=='windows-1251' ) return substr($string, $start, $length);
	
	if ( function_exists('mb_substr') ) return mb_substr($string, $start, $length, 'utf-8');
	if ( function_exists('iconv_substr') ) return iconv_substr($string, $start, $length, 'utf-8');
	
	$string = ic_uw($string);
	$string = substr($string, $start, $length);
	$string = ic_wu($string);
	
	return $string;
}

/**
 * Получить позицию символа в строке в произвольной кодировке (windows-1251, utf-8)
 * 
 * @param  string  $string
 * @param  string  $symbol
 * @return  int
 */
function mstrpos($string, $symbol) {
	$charset = detect_encoding($string);
	if ( $charset=='windows-1251' ) return strpos($string, $symbol);
	
	if ( function_exists('mb_strpos') ) return mb_strpos($string, $symbol, 0, 'utf-8');
	if ( function_exists('iconv_strpos') ) return iconv_strpos($string, $symbol, 0, 'utf-8');
	
	$string = ic_uw($string);
	$symbol = ic_uw($symbol);
	return strpos($string, $symbol);
}

/**
 * Получить последнее вхождение символа в строку в произвольной кодировке (windows-1251, utf-8)
 * 
 * @param  string  $string
 * @param  string  $symbol
 * @return  int
 */
function mstrrpos($string, $symbol) {
	$charset = detect_encoding($string);
	if ( $charset=='windows-1251' ) return strrpos($string, $symbol);
	
	if ( function_exists('mb_strrpos') ) return mb_strrpos($string, $symbol, 'utf-8');
	if ( function_exists('iconv_strrpos') ) return iconv_strrpos($string, $symbol, 'utf-8');
	
	$string = ic_uw($string);
	$symbol = ic_uw($symbol);
	return strrpos($string, $symbol);
}

/**
 * Удалить из строки символы перевода каретки и табуляции
 * 
 * @param  string  $string
 * @return  string
 */
function delete_n_t($string) {
	return str_replace(array("\r\n", "\r", "\n", "\t"), '',	$string);
}

/**
 * Преобразовать текст в массив строк
 * 
 * пустые строки не добавляются в массив
 * 
 * @param  string  $text
 * @return  array
 */
function get_list_from_text($text) {
	if ( !$text ) return array();
	
	$result = array();
	
	$strings = explode("\n", $text);
	for ($i = 0; $i < count($strings); $i++) {
		$string = $strings[$i];
		
		$string = delete_n_t($string);
		$string = trim($string);
		
		if ( $string ) $result[] = $string;
	}
	
	return $result;
}


//*****************//
// ВЫВОД В БРАУЗЕР //
//*****************//
/**
 * Отправляем текст из буфера в браузер
 * @param string $text
 */
function echo_flush($string) {
	echo $string;
	
	echo "\n";
	echo '<!--', ( str_repeat('ppp', 249) ), '-->';
	echo "\n";
	
	flush();ob_flush();
}

/**
 * Распечатываем содержимое переменной и отправляем результат из буфера в браузер
 * @param string $var
 */
function print_r_flush($var) {
	print_r($var);
	
	echo "\n";
	echo '<!--', ( str_repeat('ppp', 249) ), '-->';
	echo "\n";
	
	flush();ob_flush();
}


//********************************************//
// РАБОТА С ФАЙЛАМИ, ПАПКАМИ И ВЕБ-СТРАНИЦАМИ //
//********************************************//
/**
 * Получить все строки из файла в виде массива
 * 
 * Если файл не существует - возвращает пустой массив.
 * В конце строк символ перевода каретки отсутствует.
 * Пробелы из начала и конца строк удаляются.
 * 
 * @param   string  $filename
 * @return  array
 */
function get_list_from_file($filename) {
	
	if ( !file_exists($filename) ) return array();
	
	$list = array();
	
	$handle = fopen($filename, 'r');
	if ( !$handle ) return array();
	
	while( !flock($handle, LOCK_SH) ) usleep(50);
	
	while ( !feof($handle) ) {
		$item = fgets($handle);
		$item = trim($item);
		if ($item) $list[] = $item;
	}
	
	flock($handle, LOCK_UN);
	fclose($handle);
	
	return $list;
}

/**
 * Сохранить список в файл
 * 
 * @param   string  $filename  имя файла
 * @param   array   $list      массив строк
 * @return  bool
 */
function save_list_to_file($filename, $list) {
	
	$handle = fopen($filename, 'w');
	
	if ( flock($handle, LOCK_EX) ) {
		foreach ($list as $item) {
			$item = trim($item);
			if ( !$item ) continue;
			
			fwrite($handle, $item."\n");
		}
		flock($handle, LOCK_UN);
	}
	
	fclose($handle);
	
	return true;
}

/**
 * Дописать строку в файл
 * 
 * @param  string  $filename
 * @param  string  $text
 * @return  bool
 */
function write_to_file($filename, $text, $rewrite=false) {
	
	if ( !$filename ) return false;
	
	$mode = ( $rewrite ) ? 'w' : 'a';
	
	for ($i = 0; $i < 1000; $i++) {
		$f = fopen($filename, $mode);
		
		if ( flock($f, LOCK_EX) ) {
			fwrite($f, $text."\r\n");
			flock($f, LOCK_UN);
			fclose($f);
			break;
		} else {
			fclose($f);
			usleep(50);
		}
	}
	
	return true;
}

/**
 * Прочитать содержимое файла с учётом возможной блокировки
 *
 * @param  string  $filename
 * @return  string
 */
function read_file($filename) {
	
	clearstatcache();
	$fsize = filesize($filename);
	if ( !$fsize ) return '';
	
	$f = fopen($filename, 'r');
	while( !flock($f, LOCK_SH) ) usleep(100);
	
	clearstatcache();
	$fsize = filesize($filename);
	$content = ( $fsize ) ? fread($f, $fsize) : '';
	
	flock($f, LOCK_UN);
	fclose($f);
	
	return $content;
}

/**
 * Найти строку в файле
 * 
 * возвращает номер строки
 * если строка в файле не встречается - возвращает false 
 * 
 * поскольку нумерация строк начинается с 0, то случай неудачи надо проверять строгим равенством с false
 * 
 * @param  string  $filename
 * @param  string  $string
 * 
 * @return  int
 */
function find_in_file($filename, $string) {
	if ( !file_exists($filename) ) return false;
	
	$handle = fopen($filename, 'r');
	if ( !$handle ) return false;
	
	while( !flock($handle, LOCK_SH) ) usleep(50);
	
	$result = false;
	$num = 0;
	
	while ( !feof($handle) ) {
		$row = fgets($handle);
		$row = trim($row);
		
		if ( $row==$string ) {
			$result = $num;
			break;
		}
		
		++$num;
	}
	
	flock($handle, LOCK_UN);
	fclose($handle);
	
	return $result;
}

/**
 * Отдать файл браузеру на скачивание
 * @param string $file_url  адрес файла
 */
function download($file_url) {
	$file_title = basename($file_url);
	if ( strpos($_SERVER['HTTP_USER_AGENT'], 'MSIE')===false ) $file_title = iconv('WINDOWS-1251','UTF-8',$file_title);
	
	header('Content-Description: File Transfer');
	header('Content-Type: application/force-download');
	header('Content-Disposition: attachment; filename="' . $file_title . '"');
	
	readfile($file_url);
}

/**
 * Получить список имён файлов в указанной папке
 * 
 * @param string $dir  путь к директории
 * @return list $filenames
 */
function files_list($dir) {
	$files = array();
	
	$handle = opendir($dir);
	while( false !== ($file=readdir($handle)) ) {
		if ( $file=='.' ) continue;
		if ( $file=='..' ) continue;
		if ( is_dir($file) ) continue;
		
		$files[] = $file;
	}
	closedir($handle);
	
	return $files;
}

/**
 * Создать директорию по указанному пути
 * 
 * @param  string  $path
 * @return  bool
 */
function create_dir($path) {
	$path_arr = explode('/',$path);
	if ($path_arr[0]=='') {
		$path_arr[1] = $_SERVER['DOCUMENT_ROOT'].'/'.$path_arr[1];
		unset($path_arr[0]);
	}
	
	$path_tmp  = '';
	
	foreach($path_arr as $level) {
		if (!$level) continue;
		$path_tmp .= $level.'/';
		if ( file_exists($path_tmp) ) continue;
		mkdir($path_tmp);
	}
	
	return true;
}

/**
 * Удалить из имени файла недопустимые символы
 * @param  string  $filename
 * @return  string
 */
function clear_filename($filename) {
	$symbols = array('\\', '/', ':', '*', '?', '"', '<', '>', '|');
	
	foreach($symbols as $symbol) {
		$filename = str_replace($symbol, '', $filename);
	}
	
	return $filename;
}

/**
 * Получить расширение файла
 * 
 * Расширение определяется как часть строки после последней точки.
 *
 * @param   string  $filename
 * @return  string
 */
function get_extension($filename) {
	$filename = basename($filename);
	$parts = explode('.', $filename);
	if ( count($parts)==1 ) return '';
	return end($parts);
}

/**
 * Получить содержимое страницы с указанной паузой
 * 
 * @param string $url    адрес страницы
 * @param int    $sleep  пауза в секундах
 * @return  string
 */
function get_html($url, $sleep=0) {
	if ( !$url ) return '';
	if ($sleep) sleep($sleep);
	$html = file_get_contents($url);
	return $html;
}

/**
 * Получить содержимое страницы
 * 
 * @param string $url    адрес страницы
 * @return  string
 */
function get_content($url) {
	
	$ch = curl_init($url);
	
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 30);
	curl_setopt($ch, CURLOPT_TIMEOUT, 30);
	curl_setopt ($ch, CURLOPT_USERAGENT, "User-Agent: Mozilla/5.0 (Windows NT 6.1; rv:2.0) Gecko/20100101 Firefox/4.0");
	
	$res = curl_exec($ch);
	$error = curl_errno($ch);
	curl_close($ch);
	
	return ( $error===0 ) ? $res : false;
}

/**
 * Проверить наличие связи
 * 
 * @return  bool
 */
function ping() {
	$ch = curl_init('http://ya.ru');
	
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 300);
	curl_setopt($ch, CURLOPT_TIMEOUT, 300);
	curl_setopt ($ch, CURLOPT_USERAGENT, "User-Agent: Mozilla/5.0 (Windows NT 6.1; rv:2.0) Gecko/20100101 Firefox/4.0");
		
	curl_exec($ch);
	$error = curl_errno($ch);
	
	curl_close($ch);
	
	return ( $error===0 ) ? true : false;
}


//**************************//
// ХРАНЕНИЕ ДАННЫХ В ФАЙЛАХ //
//**************************//
function get_items_from_store($file) {
	if ( !$file ) return array();
	if ( !file_exists($file) ) return array();
	
	$items = file_get_contents($file);
	$items = unserialize($items);
	
	if ( !is_array($items) ) return array();
	
	return $items;
}

function get_item_from_store($file, $item_id) {
	if ( !$file ) return false;
	if ( !file_exists($file) ) return false;

	$items = file_get_contents($file);
	$items = unserialize($items);

	if ( !is_array($items) ) return false;
	if ( !isset($items[$item_id]) ) return false;
	
	$item = $items[$item_id];
	
	if ( $item=='false' ) return false;
	if ( $item=='true' ) return true;
	return $item;
}

function add_item_to_store($file, $item, $item_id='') {
	if ( !$file ) return false;
	
	if ( file_exists($file) ) {
		$items = file_get_contents($file);
		$items = unserialize($items);
		
		if ( !is_array($items) ) return false;
	} else {
		$items = array();
	}
	
	if ( $item_id ) {
		$items[ $item_id ] = $item;
	} else {
		$items[] = $item;
		$item_id = end(array_keys($items));
	}
	
	$items = serialize($items);
	write_to_file($file, $items, true);
	
	return $item_id;
}

function delete_item_from_store($file, $item_id) {
	if ( !$file ) return false;
	if ( !file_exists($file) ) return false;

	$items = file_get_contents($file);
	$items = unserialize($items);

	if ( !is_array($items) ) return false;
	if ( !isset($items[$item_id]) ) return false;

	unset($items[$item_id]);

	$items = serialize($items);
	write_to_file($file, $items, true);

	return true;
}

function find_item_in_store($file, $item) {
	if ( !$file ) return false;
	if ( !file_exists($file) ) return false;

	$items = file_get_contents($file);
	$items = unserialize($items);

	if ( !is_array($items) ) return false;

	foreach ($items as $cur_item) {
		if ( $cur_item==$item ) return true;
	}

	return false;
}

function update_item_in_store($file, $item_id, $item) {
	if ( !$file ) return false;
	if ( !file_exists($file) ) write_to_file($file, serialize(array()), true);
	
	$items = file_get_contents($file);
	$items = unserialize($items);
	
	if ( !is_array($items) ) return false;
	
	$items[$item_id] = $item;
	write_to_file($file, serialize($items), true);
	
	return true;
}


//************************//
// ХРАНЕНИЕ ДАННЫХ В БАЗЕ //
//************************//
/**
 * Получить записи из базы с учётом всех зависимостей
 * 
 * @param  string  $tbl_name
 * @return  array
 */
function get_rows($tbl_name) {
	global $database;
	
	$result = array();
	
	$rows = $database->select("SELECT * FROM $tbl_name");
	foreach ($rows as $row) {
		$result[ $row['id'] ] = $row;
		
		foreach ($row as $field => $value) {
			if ( substr($field, -3)=='_id' ) {
				$tbl = substr($field, 0, strlen($field)-3);
				$obj = $database->selectRow("SELECT * FROM ".$tbl."s WHERE id=?d", $value);
				$result[ $row['id'] ][$tbl] = $obj;
			} else {
				$uns = unserialize($value);
				if ( $uns && is_array($uns) ) {
					$a = array();
					foreach ($uns as $uid) {
						$o = $database->selectRow("SELECT * FROM $field WHERE id=?d", $uid);
						$a[$uid] = $o;
					}
					$result[ $row['id'] ][$field] = $uns;
					$result[ $row['id'] ][$field.'_array'] = $a;
				}
			}
		}
	}
	
	return $result;
}

/**
 * Получить запись из базы с учётом всех зависимостей
 * 
 * @param  string  $tbl_name
 * @param  int  $id
 * 
 * @return  array
 */
function get_row($tbl_name, $id) {
	global $database;
	
	$row = $database->selectRow("SELECT * FROM $tbl_name WHERE id=?d", $id);
	foreach ($row as $field => $value) {
		if ( substr($field, -3)=='_id' ) {
			$tbl = substr($field, 0, strlen($field)-3);
			$obj = $database->selectRow("SELECT * FROM ".$tbl."s WHERE id=?d", $value);
			$row[$tbl] = $obj;
		} else {
			$uns = unserialize($value);
			if ( $uns && is_array($uns) ) {
				$a = array();
				foreach ($uns as $uid) {
					$o = $database->selectRow("SELECT * FROM $field WHERE id=?d", $uid);
					$a[$uid] = $o;
				}
				$row[$field] = $uns;
				$row[$field.'_array'] = $a;
			}
		}
	}
	
	return $row;
}


//**********************//
// РАБОТА С НАСТРОЙКАМИ //
//**********************//
/**
 * Сохранить значение параметра настроек
 *
 * если в качестве значения передан массив - он сериализуется для записи в базу
 *
 * @param  string  $name
 * @param  string|array  $value
 * @return boolean
 */
function save_settings($name, $value) {
	global $database;

	if ( is_array($value) ) $value = serialize($value);

	$id = $database->selectCell("SELECT id FROM settings WHERE name=?", $name);
	if ( $id ) {
		$database->query("UPDATE settings SET value=? WHERE id=?d", $value, $id);
	} else {
		$database->query("INSERT INTO settings SET name=?, value=?", $name, $value);
	}

	return true;
}

/**
 * Получить значение параметра настроек
 *
 * если такого параметра в настройках нет - возвращает $default
 * если в качестве $default передан массив - значение параметра десериализуется
 *
 * @param  string  $name
 * @param  mixed  $default
 *
 * @return  mixed
 */
function get_settings($name, $default=false) {
	global $database;

	$id = $database->selectCell("SELECT id FROM settings WHERE name=?", $name);
	if ( !$id ) return $default;

	$value = $database->selectCell("SELECT value FROM settings WHERE name=?", $name);
	if ( is_array($default) ) {
		$value = unserialize($value);
		if ( !$value ) $value = array();
	}
	return $value;
}

/**
 * Удалить параметр настроек из базы
 *
 * @param  string  $name
 */
function delete_settings($name) {
	global $database;
	$database->query("DELETE FROM settings WHERE name=?", $name);
}


//***********//
// КОДИРОВКА //
//***********//
/**
 * Определить кодировку строки
 * 
 * Функция распознаёт кодировку строк в utf-8 и windows-1251.
 * В противном случае возвращает null.
 * 
 * @param   string  $string     строка
 * @return  string  $encoding  'utf-8', 'windows-1251', null
 */
function detect_encoding($string) {  
	static $list = array('utf-8', 'windows-1251');
	
	foreach ($list as $item) {
		$sample = iconv($item, $item, $string);
		if (md5($sample) == md5($string)) return $item;
	}
	
	return null;
}

/**
 * Перекодировать текст из UTF-8 в WINDOWS-1251
 * 
 * @param  string  $text
 * @return  string
 */
function ic_uw($text) {
	return iconv('UTF-8', 'WINDOWS-1251//IGNORE', $text);
}

/**
 * Перекодировать текст из WINDOWS-1251 в UTF-8
 * 
 * @param  string  $text
 * @return  string
 */
function ic_wu($text) {
	return iconv('WINDOWS-1251', 'UTF-8', $text);
}

/**
 * Удалить из UTF-8 строки все символы, не входящие в WINDOWS-1251
 * 
 * @param  string  $text
 * @return  string
 */
function ic_s($text) {
	$text = iconv('UTF-8', 'WINDOWS-1251//IGNORE', $text);
	$text = iconv('WINDOWS-1251', 'UTF-8', $text);
	return $text;
}

/**
 * Костыль для корректной работы json с кириллицей
 * @param string $var
 */
function json_safe_encode($var) { // основная
   return json_encode( json_fix_cyr($var) );
}
function json_fix_cyr($var) { // служебная
   if (is_array($var)) {
       $new = array();
       foreach ($var as $k => $v) {
           $new[json_fix_cyr($k)] = json_fix_cyr($v);
       }
       $var = $new;
   } elseif (is_object($var)) {
       $vars = get_object_vars($var);
       foreach ($vars as $m => $v) {
           $var->$m = json_fix_cyr($v);
       }
   } elseif (is_string($var)) {
       $var = iconv('cp1251', 'utf-8', $var);
   }
   return $var;
}

/**
 * Преобразовать в json-кодированной строке коды в русские буквы
 * 
 * @param  string  $str
 * @return  string
 */
function utf8_JSON_russian_cyr($str){
	$arr_replace_symbols = array(
			'\u0410' => 'А', '\u0430' => 'а',
			'\u0411' => 'Б', '\u0431' => 'б',
			'\u0412' => 'В', '\u0432' => 'в',
			'\u0413' => 'Г', '\u0433' => 'г',
			'\u0414' => 'Д', '\u0434' => 'д',
			'\u0415' => 'Е', '\u0435' => 'е',
			'\u0401' => 'Ё', '\u0451' => 'ё',
			'\u0416' => 'Ж', '\u0436' => 'ж',
			'\u0417' => 'З', '\u0437' => 'з',
			'\u0418' => 'И', '\u0438' => 'и',
			'\u0419' => 'Й', '\u0439' => 'й',
			'\u041a' => 'К', '\u043a' => 'к',
			'\u041b' => 'Л', '\u043b' => 'л',
			'\u041c' => 'М', '\u043c' => 'м',
			'\u041d' => 'Н', '\u043d' => 'н',
			'\u041e' => 'О', '\u043e' => 'о',
			'\u041f' => 'П', '\u043f' => 'п',
			'\u0420' => 'Р', '\u0440' => 'р',
			'\u0421' => 'С', '\u0441' => 'с',
			'\u0422' => 'Т', '\u0442' => 'т',
			'\u0423' => 'У', '\u0443' => 'у',
			'\u0424' => 'Ф', '\u0444' => 'ф',
			'\u0425' => 'Х', '\u0445' => 'х',
			'\u0426' => 'Ц', '\u0446' => 'ц',
			'\u0427' => 'Ч', '\u0447' => 'ч',
			'\u0428' => 'Ш', '\u0448' => 'ш',
			'\u0429' => 'Щ', '\u0449' => 'щ',
			'\u042a' => 'Ъ', '\u044a' => 'ъ',
			'\u042d' => 'Ы', '\u044b' => 'ы',
			'\u042c' => 'Ь', '\u044c' => 'ь',
			'\u042d' => 'Э', '\u044d' => 'э',
			'\u042e' => 'Ю', '\u044e' => 'ю',
			'\u042f' => 'Я', '\u044f' => 'я',
			'\u003c' => '<'
	);

	foreach($arr_replace_symbols as $from_gluk => $to_normal){
		$str = str_replace ($from_gluk,$to_normal,$str);
	}

	return $str;
}


//*******************//
// ДАННЫЕ ИЗ ЗАПРОСА //
//*******************//
/**
 * Проверить, существует ли переменная в запросе
 * 
 * Если указано not_empty - осуществляет проверку на непустоту 
 * Если указана value - проверяет на равенство этому значению
 * 
 * @param  string  $var_name   имя параметра
 * @param  bool    $not_empty  параметр должен быть непустым
 * @param  string  $value      параметр должен быть равен этому значению
 * @return  bool
 */
function isset_req_var($var_name, $not_empty=true, $value='') {
	if ( !$var_name && !is_string($var_name) ) return false;
	if ( !isset($_REQUEST[$var_name]) ) return false;
	if ( $not_empty && !$_REQUEST[$var_name] ) return false;
	if ( $value!='' && $_REQUEST[$var_name]!=$value ) return false;
	return true;
}

/**
 * Получить значение параметра из запроса
 * 
 * Если параметр в запросе есть и он непустой - возвращает его значение
 * Иначе возвращает значение, указанное в default
 * 
 * @param  string  $var_name  имя параметра
 * @param  mixed   $default   значение, которое надо вернуть, если параметр в запросе отсутствует (если передан массив значений - к этому виду будет приведён каждый элемент массива)
 * @param  string  $type      тип передаваемых данных (json - кодированные в json-формате, array - массив значений, bool - вернуть 1 или 0, list - преобразовать текст в массив строк)
 * @return  mixed
 */
function get_req_var($var_name, $default='', $type='') {
	if ( !$var_name ) return $default;
	if ( !isset($_REQUEST[$var_name]) ) return $default;
	
	$value = $_REQUEST[$var_name];
	
	if ( $type=='array' && is_array($value) ) {
		foreach ($value as $k=>$v) {
			if ( get_magic_quotes_gpc() ) $v = stripslashes($v);
			$v = urldecode($v);
			$value[$k] = $v;
		}
	} else {
		if ( get_magic_quotes_gpc() ) $value = stripslashes($value);
		$value = urldecode($value);
	}
	
	if ( is_numeric($default) ) {
		if ( is_array($value) ) {
			foreach ($value as $key => $val) {
				$val = intval($val);
				if ( !$val ) $val = 0;
				$value[$key] = $val;
			}
		} else {
			$value = intval($value);
			if ( !$value ) $value = 0;
		}
	}
	
	if ( $value==='0' || $value===0 ) return 0;
	
	if ( $type=='json' ) $value = json_decode($value, true);
	if ( $type=='bool' ) $value = ($value) ? 1 : 0;
	if ( $type=='list' ) $value = get_list_from_text($value);
	
	return ($value) ? $value : $default;
}


//***********//
// АЛГОРИТМЫ //
//***********//
/**
 * Сгенерировать все возможные комбинации символов до указанной длины
 * 
 * @param  array  $syms    массив символов
 * @param  int    $length  максимальная длина генерируемых строк
 * @return  array
 */
function comb($syms, $length) {
	$list = array();
	
	foreach($syms as $sym) $list[] = $sym;
	
	for($i=1;$i<$length;$i++) {
		$new_list = array();
		foreach($list as $item) {
			foreach($syms as $sym) $new_list[] = $item.$sym;
		}
		$list = array_merge($list, $new_list);
	}
	
	return $list;
}

function generate($length = 12){
	$chars = 'abdefhiknrstyzABDEFGHKNQRSTYZ23456789';
	$numChars = strlen($chars);
	$string = '';
	for ($i = 0; $i < $length; $i++) {
	  $string .= substr($chars, rand(1, $numChars) - 1, 1);
	}
	return $string;
}

/**
 * Преобразовать дерево в список
 * 
 * @param  array   $array     массив
 * @param  string  $nodeItem  имя элемента, в котором хранятся подчинённые элемента 
 * @param  int     $level     текущий уровень вложенности (будет добавлен в элементы результирующего массива как level - начиная с 0 или указанного значения)
 * @return  array
 */ 
function array_to_list($array, $nodeItem='childNodes', $level=0) {
	
	$list = array();
	
	foreach ($array as $item) {
		$item['level'] = $level;
		$list[] = $item;
		if ( isset($item[$nodeItem]) ) {
			$sublist = array_to_list($item[$nodeItem], $nodeItem, $level+1);
			$list = array_merge($list, $sublist);
		}
	}
	
	return $list;
}

/**
 * Преобразовать список, представляющий дерево, в отсортированный массив
 * 
 * @param  array  $array
 * @param  string  $id_index
 * @param  string  $parent_index
 * @param  string  $title_index
 */
function tree_to_list($array, $id_index='id', $parent_index='parent_id', $title_index='title') {
	
	$result = array();
	
	foreach ($array as $item) {
		if ( find_in_array($array, array($id_index => $item[$parent_index])) ) continue;
		
		$result[] = $item;
		$childs = tree_to_list_node($array, $id_index, $parent_index, $title_index, $item[$id_index], 1);
		
		$result = array_merge($result, $childs);
	}
	
	return $result;
}
function tree_to_list_node($array, $id_index, $parent_index, $title_index, $parent_id, $level) {
	
	if ( $level>1000 ) return array();
	
	$result = array();
	$prefix = str_repeat('--', $level);
	
	foreach ($array as $item) {
		if ( $item[$parent_index]!=$parent_id ) continue;
		
		if ( $title_index ) $item[ $title_index ] = $prefix . ' ' . $item[ $title_index ];
		
		$result[] = $item;
		$childs = tree_to_list_node($array, $id_index, $parent_index, $title_index, $item[$id_index], $level+1);
		
		$result = array_merge($result, $childs);
	}
	
	return $result;
}

/**
 * Поменять значения переменных
 * 
 * @param  mixed  $var1
 * @param  mixed  $var2
 * @return  none
 */
function change_values(&$var1, &$var2) {
	$t = $var1;
	$var1 = $var2;
	$var2 = $t;
}

/**
 * Получить случайную дату из указанного диапазона
 * Все даты в формате YYYY-MM-DD HH:II:SS
 * 
 * @param  date  $date_start
 * @param  date  $date_end
 * @return  datetime
 */
function rand_date($date_start, $date_end) {
	$time_start = strtotime($date_start);
	$time_end = strtotime($date_end);
	
	$time = rand($time_start, $time_end);
	
	return date('Y-m-d H:i:s', $time);
}

/**
 * Добавить элемент в массив по указанному пути
 * 
 * @param  array   $array
 * @param  array   $path
 * @param  mixed   $elem
 * @param  string  $elem_name
 * 
 * @return  array
 */
function add_to_array($array, $path, $elem, $elem_name='') {
	$sub = &$array;
	
	foreach ($path as $item) {
		if ( !array_key_exists($item, $sub) ) $sub[ $item ] = array();
		$sub = &$sub[$item];
	}
	
	if ( $elem_name )
		$sub['link'] = $elem;
	else
		$sub = $elem;
	
	return $array;
}

/**
 * Обработать указанные адреса в multi_curl с указанным числом потоков
 * 
 * @param  array  $urls
 * @param  int  $flowcount
 */ 
function Parse($urls, $flowcount) {
	
	// проверяем поступившие данные
	if ( !$urls ) return true;
	if ( count($urls) < $flowcount ) $flowcount = count($urls);
	
	// формируем массив curl-сессий для всех потоков
	$ch = array();
	foreach ($urls as $url) {
		// создадим указатель curl на поток
		$ch = curl_init( $url );
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		
		// добавим указатель в массив
		$chs[] = $ch;
	}
	
	// формируем первую партию
	$mh = curl_multi_init();
	for ($i=0; $i<$flowcount; $i++) curl_multi_add_handle($mh, array_shift($chs));
	
	// исполняем потоки
	$flows = null;
	do {
		do {
			curl_multi_exec($mh, $flows);
			sleep(10);
		} while( $flows==$flowcount );
		
		$info = curl_multi_info_read($mh);
			
		// сохраняем ответ потока в логах
		$content = curl_multi_getcontent($info['handle']);
		if ( $content ) {
			$logname = 'parse_log_' . date('Y-m-d-h-i-s') . '_' . get_uniq_id() . '.txt';
			write_to_file(LOG.'/'.$logname, $content);
		}
			
		curl_multi_remove_handle($mh, $info['handle']);
		curl_close($info['handle']);
			
		if ( count($chs)>0 ) {
			curl_multi_add_handle($mh, array_shift($chs));
			curl_multi_exec($mh, $flows);
		} else {
			$flowcount--;
		}
			
	} while( $flows>0 );
	
	curl_multi_close($mh);
	
	return true;
}

/**
 * Найти элемент в массиве, в котором указанные поля совпадают с указанными значениями
 * 
 * @param  array  $array
 * @param  array  $fields
 * @param  string  $return_field
 * @return  array
 */
function find_in_array($array, $fields, $return_field='') {
	
	if ( !is_array($array) ) return false;
	
	foreach ($array as $item) {
		
		$same = true;
		
		foreach ($fields as $field=>$value) {
			
			if ( !isset($item[$field]) ) {
				$same = false;
				break;
			}
			
			if ( $item[$field]!=$value ) {
				$same = false;
				break;
			}
			
		}
		
		if ( $same ) {
			if ( $return_field && isset($item[$return_field]) ) return $item[$return_field];
			return $item;
		}
	}
	
	return false;
}

/**
 * Найти элемент в массиве и вернуть только уникальные
 */
function search_in_array($needle, $array) {
    foreach($array as $value)
		if (in_array($needle, $value)) return true;
    return false;
}

/**
 * Групировать и сложить по ключу title
 */
function sum_by_array_title($arr) {
	if( !is_array($arr) ) return array();
	
	foreach ($arr as $l) $res[$l['title']][] = (array_key_exists('data', $l['data'])) ? $l['data']['data'] : '';
	foreach ($res as $k) $result[] = array_sum($k);
	
	return $result;
}


/**
 * Проверить сообщения на уникальность по поиску яндекса и удалить неуникальные
 * 
 * поиск идёт по сайтам yandex.ry, qip.ru и mail.ru
 * для этого используется глобальный объект $ya (должен быть объявлен и настроен заранее)
 * 
 * текст должен храниться в поле text у сообщения
 * 
 * @param  array  $messages
 * @param  int  $ya_cnt
 * @return  array
 */
function talks_uniq($messages, $ya_cnt=5) {
	
	if ( $ya_cnt<=0 ) return $messages;
	
	global $ya;
	
	$parser = new Parser();
	$result = array();
	$tmp = array();
	$cnt = 0;
	
	foreach ($messages as $message) {
		
		if ( count($tmp)<$ya_cnt || mstrlen($message['text'])<70 ) {
			
			$result[ $cnt ] = $message;
			$tmp[] = $cnt;
			++$cnt;
			continue;
			
		} else {
			
			$str = $parser->clear_text($message['text']);
			$str = cut_string($str, 70, array(' ',',','.','!','?','-','"',"'"), false);
			$str = '"'.$str.'"';
			
			$ya_tmp = rand(0, 30);
			if ( $ya_tmp<10 ) $src = 'ya';
			elseif ( $ya_tmp<20 ) $src = 'ra';
			else $src = 'qip';
			
			$res = $ya->count_results($str, $src);
			if ( $res ) {
				foreach ($tmp as $id) unset($result[$id]);
			} else {
				$result[ $cnt ] = $message;
			}
			
			$tmp = array();
			++$cnt;
		}
	}
	
	$res = array();
	foreach ($result as $message) $res[] = $message;
	
	return $res;
}

/**
 * Приостановить выполнение всех остальных процессов
 *
 * @param  string  $id
 * @return resource
 */
function block($id) {
	
	$filename = LOG . '/' . md5($id) . '.txt';
	$h = fopen($filename, 'w+');
	while( !flock($h, LOCK_EX) ) usleep(50);
	
	return $h;
}

/**
 * Разрешить выполнение других процессов
 *
 * @param  resource  $h
 */
function unblock($h) {
	if ( !$h ) return false;
	flock($h, LOCK_UN);
	fclose($h);
}

/**
 * Записать сообщение в лог задания
 * 
 * для многопоточных процессов, организованных не через Threads
 * 
 * @param  string  $string
 * @param  string  $uniqid
 */
function logger($string, $uniqid) {
	$uniqid = clear_filename($uniqid);
	write_to_file(LOG."/$uniqid.txt", $string);
}

/**
 * Остановить задание
 * 
 * для многопоточных процессов, организованных не через Threads
 * 
 * @param  string  $uniqid
 */
function stop($uniqid) {
	$uniqid = clear_filename($uniqid);
	
	if ( file_exists(LOG."/stop_$uniqid.txt") ) {
		logger('Работа остановлена!', $uniqid);
		die("Работа остановлена!");
	}
}
